/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : group.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 13-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char cvers[LEN+1];
extern int cur_groupnum;
extern int last_resp;		/* page.c */
extern int this_resp;		/* page.c */
extern int space_mode;		/* select.c */

char *glob_group;
int index_point;
int first_subj_on_screen;
int last_subj_on_screen;


#ifdef SIGTSTP
void group_susp (sig)
	int sig;
{
	Raw(FALSE);
	putchar('\n');
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask(sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
	kill(0, SIGTSTP);

	signal(SIGTSTP, group_susp);

	if (! update) {
		Raw(TRUE);
	}
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	mail_setup();
	show_group_page(glob_group);
}
#endif


void group_page (group)
	char *group;
{
	char buf[LEN+1];
	char group_path[LEN+1];
	char ch;
	char *p;
	int flag, i, n;
	int kill_state;
	long old_artnum;
	int old_top;
	int ret1, ret2;
	int sav_groupnum;
	int thread_marked_unread = FALSE;

	glob_group = group;
	sav_groupnum = cur_groupnum;
	
	strcpy (group_path, group);			/* turn comp.unix.amiga into */
	for (p = group_path; *p; p++)		/* comp/unix/amiga */
		if (*p == '.')
			*p = '/';

	last_resp = -1;
	this_resp = -1;
	index_group (group, group_path);	/* update index file */
	read_newsrc_line (group);			/* get sequencer information */

	if (space_mode) {
		for (i = 0; i < top_base; i++) {
			if (new_responses (i)) {
				break;
			}
		}
		if (i < top_base) {
			index_point = i;
		} else {
			index_point = top_base - 1;
		}
	} else {
		index_point = top_base - 1;
	}
	
	clear_note_area ();

	show_group_page (group);

	while (1) {
		ch = ReadCh();

		if (ch > '0' && ch <= '9') {	/* 0 goes to basenote */
			prompt_subject_num(ch, group);
		} else switch (ch) {
			case '!':
				shell_escape ();
				show_group_page (group);
				break;

			case '$':	/* show last page of articles */
end_of_list:			
				index_point = top_base - 1;
				show_group_page (group);
				break;
				
			case '-':	/* go to last viewed article */
				if (this_resp < 0) {
					info_message(txt_no_last_message);
					break;
				}
				index_point = show_page (this_resp, group, group_path);
				if (index_point < 0) {
					space_mode = FALSE;
					goto group_done;
				}
				clear_note_area ();
				show_group_page (group);
				break;

			case '/':	/* forward/backward search */
			case '?':
				i = (ch == '/');
				search_subject (i, group);
				break;

			case '\r':
			case '\n':	/* read current basenote */
				if (index_point < 0) {
					info_message(txt_no_arts);
					break;
				}
				i = (int) base[index_point];
				index_point = show_page(i, group, group_path);
				if (index_point < 0) {
					space_mode = FALSE;
					goto group_done;
				}
				clear_note_area ();
				show_group_page (group);
				break;

			case '\t':
				space_mode = TRUE;

				if (index_point < 0
				|| (n=next_unread((int) base[index_point]))<0) {
					for (i = cur_groupnum+1;
							i < local_top; i++)
						if (unread[i] > 0)
							break;
					if (i >= local_top)
						goto group_done;

					cur_groupnum = i;
					index_point = -3;
					goto group_done;
				}
				index_point = show_page(n, group, group_path);
				if (index_point < 0)
					goto group_done;
				clear_note_area ();
				show_group_page(group);
				break;
	
			case 27:	/* common arrow keys */
				ch = ReadCh();
				if (ch == '[' || ch == 'O')
					ch = ReadCh();
				switch (ch) {
				case 'A':
				case 'D':
					goto group_up;

				case 'B':
				case 'C':
					goto group_down;

				case 'G':		/* ansi  PgDn */
				case 'U':		/* at386 PgDn */
					goto group_page_down;

				case 'I':		/* ansi  PgUp */
				case 'V':		/* at386 PgUp */
					goto group_page_up;

				case 'H':		/* at386  Home */
					index_point = 0;
					show_group_page (group);
					break;
					
				case 'F':		/* ansi  End */
				case 'Y':		/* at386  End */
					goto end_of_list;
				}
				break;

			case ctrl('D'):		/* page down */
			case ' ':	
group_page_down:
				if (!top_base || index_point == top_base - 1)
					break;

				erase_subject_arrow();
				index_point += NOTESLINES / 2;
				if (index_point >= top_base)
					index_point = top_base - 1;

				if (index_point < first_subj_on_screen
				|| index_point >= last_subj_on_screen)
					show_group_page(group);
				else
					draw_subject_arrow();
				break;

			case ctrl('K'):		/* kill article */
 				if (index_point < 0) {
 					info_message (txt_no_arts);
					break;
				}
				if (kill_articles) {
					old_top = top;
					n = base[index_point];
					old_artnum = arts[n].artnum;
					if (kill_art_menu (group, (int) base[index_point])) {
						kill_any_articles (group);
						reload_index_file (group, TRUE);
						index_point = find_new_pos (old_top, old_artnum, index_point);
					}
					show_group_page (group);
				} else {
					info_message (txt_switch_on_kill_art_menu);
				}
				break;

			case ctrl('L'):		/* return to index */
			case ctrl('R'):
			case ctrl('W'):
#ifndef USE_CLEARSCREEN
				ClearScreen ();
#endif
				show_group_page(group);
				break;

			case ctrl('N'):
			case 'j':		/* line down */
group_down:
				if (!top_base || index_point + 1 >= top_base)
					break;

				if (index_point + 1 >= last_subj_on_screen) {
#ifndef USE_CLEARSCREEN
					erase_subject_arrow();
#endif					
					index_point++;
					show_group_page(group);
				} else {
					erase_subject_arrow();
					index_point++;
					draw_subject_arrow();
				}
				break;

			case ctrl('P'):
			case 'k':		/* line up */
group_up:
				if (!top_base || !index_point)
					break;

				if (index_point <= first_subj_on_screen) {
					index_point--;
					show_group_page(group);
				} else {
					erase_subject_arrow();
					index_point--;
					draw_subject_arrow();
				}
				break;

			case ctrl('U'):		/* page up */
			case 'b':
group_page_up:
				if (!top_base)
					break;

#ifndef USE_CLEARSCREEN
				clear_message ();
#endif
				erase_subject_arrow();
				index_point -= NOTESLINES / 2;
				if (index_point < 0)
					index_point = 0;
				if (index_point < first_subj_on_screen
				|| index_point >= last_subj_on_screen)
					show_group_page(group);
				else
					draw_subject_arrow();
				break;

			case 'a':	/* author search forward */
			case 'A':	/* author search backward */
				if (index_point < 0) {
					info_message(txt_no_arts);
					break;
				}

				i = (ch == 'a');

				n = search_author((int) base[index_point], i);
				if (n < 0)
					break;

				index_point = show_page(n, group, group_path);
				if (index_point < 0) {
					space_mode = FALSE;
					goto group_done;
				}
				clear_note_area ();
				show_group_page (group);
				break;

			case 'B':	/* bug/gripe/comment mailed to author */
				mail_bug_report ();
#ifndef USE_CLEARSCREEN
				ClearScreen ();
#endif
				show_group_page (group);
				break;
				
			case 'c':	/* catchup--mark all articles as read */
			    if (prompt_yn (LINES, txt_mark_all_read, 'y')) {
					for (n = 0; n < top; n++) {
						arts[n].unread = ART_READ;
					}
					if (cur_groupnum + 1 < local_top) {
						cur_groupnum++;
					}
					goto group_done;
			    }
			    break;

			case 'g':	/* choose a new group by name */
				n = choose_new_group ();
				if (n >= 0 && n != cur_groupnum) {
					cur_groupnum = n;
					index_point = -3;
					goto group_done;
				}
				break;

			case 'h':
				help_group_commands ();
				show_group_page (group);
				break;

			case 'H':
				help_group_info ();
				show_group_page (group);
				break;

			case 'I':	/* toggle inverse video */
				inverse_okay = !inverse_okay;
				if (inverse_okay)
					info_message(txt_inverse_on);
				else
					info_message(txt_inverse_off);
				show_group_page(group);
				break;

			case 'K':	/* mark rest of thread as read */
				if (index_point < 0) {
					info_message (txt_no_next_unread_art);
					break;
				}
				if (new_responses (index_point)) {
				    for (i = base[index_point]; i >= 0; i = arts[i].thread)
						arts[i].unread = 0;
					if (draw_arrow_mark) {
					    MoveCursor(INDEX_TOP + (index_point - first_subj_on_screen), 8);
						putchar(' ');
						MoveCursor (LINES, 0);
					    fflush(stdout);
					} else {
						i =	index_point-first_subj_on_screen;
						screen[i].col[8] = ' ';
						draw_subject_arrow();
					}
				    flag = FALSE;
				} else
				    flag = TRUE;

				n = next_unread (next_response ((int) base[index_point]));
				if (n < 0) {
				    if (flag)
						info_message (txt_no_next_unread_art);
				    else
						MoveCursor (LINES, 0);
				    break;
				}

				if ((n = which_base(n)) < 0) {
					info_message("Internal error: K which_base < 0");
					break;
				}

				if (n >= last_subj_on_screen) {
					index_point = n;
					show_group_page(group);
				} else {
					erase_subject_arrow();
					index_point = n;
					draw_subject_arrow();
				}
				break;

			case 'M':	/* options menu */
				old_top = top;
				n = base[index_point];
				old_artnum = arts[n].artnum;
				n = sort_art_type;
				kill_state = change_rcfile (group, TRUE);
				if (kill_state == NO_KILLING && n != sort_art_type) {
					make_threads (TRUE);
					find_base ();
				}
				index_point = find_new_pos (old_top, old_artnum, index_point);
				show_group_page (group);
			    break;

			case 'n':	/* next group */
				clear_message();
				if (cur_groupnum + 1 >= local_top)
					info_message(txt_no_more_groups);
				else {
					cur_groupnum++;
					index_point = -3;
					space_mode = FALSE;
					goto group_done;
				}
				break;

			case 'N':	/* go to next unread article */
				if (index_point < 0) {
					info_message(txt_no_next_unread_art);
					break;
				}

				n = next_unread( (int) base[index_point]);
				if (n == -1)
					info_message(txt_no_next_unread_art);
				else {
					index_point = show_page(n, group, group_path);
					if (index_point < 0) {
						space_mode = FALSE;
						goto group_done;
					}
					clear_note_area ();
					show_group_page(group);
				}
				break;

			case 'p':	/* previous group */
				clear_message();
				if (cur_groupnum <= 0)
					info_message(txt_no_prev_group);
				else {
					cur_groupnum--;
					index_point = -3;
					space_mode = FALSE;
					goto group_done;
				}
				break;

			case 'P':	/* go to previous unread article */
				if (index_point < 0) {
				    info_message(txt_no_prev_unread_art);
				    break;
				}
				n = prev_response( (int) base[index_point]);
				n = prev_unread(n);
				if (n == -1)
				    info_message(txt_no_prev_unread_art);
				else {
					index_point = show_page (n, group, group_path);
					if (index_point < 0) {
						space_mode = FALSE;
						goto group_done;
					}
					clear_note_area ();
					show_group_page(group);
				}
				break;

			case 'q':		/* quit */
				index_point = -2;
				space_mode = FALSE;
				goto group_done;

			case 's':	/* subscribe to this group */
				subscribe(group, ':', my_group[cur_groupnum], TRUE);
				sprintf(buf, txt_subscribed_to, group);
				info_message(buf);
				break;

			case 'S':	/* save regex pattern to file/s */
				n = save_regex_arts (group_path, group);
				ret1 = post_process_files ();
				ret2 = untag_all_articles ();
				if (ret1 || ret2 || n) {
					show_group_page (group);
				}
				free_save_array ();
				break;
			
			case 't':
			case 'i':	/* return to group selection page */
				goto group_done;

			case 'T':	/* tag/untag article for saving */
 				if (index_point >= 0) {
					if (arts[base[index_point]].tagged) {
						arts[base[index_point]].tagged = 0;
						show_group_page (group);
						info_message ("untagged");
					} else {
						arts[base[index_point]].tagged = ++num_of_tagged_files;
						show_group_page (group);
						info_message ("tagged for saving");
					}
				}
				break;

			case 'u':	/* unsubscribe to this group */
				subscribe(group, '!', my_group[cur_groupnum], TRUE);
				sprintf(buf, txt_unsubscribed_to, group);
				info_message(buf);
				break;

			case 'U':	/* untag all articles */
 				if (index_point >= 0) {
					untag_all_articles ();
					show_group_page (group);
				}
				break;

			case 'v':
				info_message (cvers);
				break;

			case 'w':	/* post a basenote */
				post_base (group);
				update_newsrc (group, my_group[cur_groupnum]);
				index_group (group, group_path);
				read_newsrc_line (group);
				index_point = top_base - 1;
				show_group_page (group);
				break;

			case 'W':	/* display messages posted by user */
				if (user_posted_messages ()) {
					show_group_page(group);
				}
				break;

			case 'z':	/* mark article as unread (to return) */
 				if (index_point < 0) {
 					info_message (txt_no_arts);
					break;
				}
				i = base[index_point];
				arts[i].unread = ART_UNREAD;
				i = index_point-first_subj_on_screen;
				screen[i].col[8] = '+';
				if (draw_arrow_mark) {
				    MoveCursor(INDEX_TOP + (index_point - first_subj_on_screen), 8);
					putchar ('+');
				} else {
					draw_subject_arrow();
				}
				info_message(txt_art_marked_as_unread);
				break;

			case 'Z':	/* mark thread as unread */
 				if (index_point < 0) {
 					info_message (txt_no_arts);
					break;
				}
				for (i = base[index_point] ; i != -1 ; i = arts[i].thread) {
					arts[i].unread = ART_UNREAD;
					thread_marked_unread = TRUE;
				}
				if (thread_marked_unread) {
					i = index_point-first_subj_on_screen;
					screen[i].col[8] = '+';
					if (draw_arrow_mark) {
					    MoveCursor(INDEX_TOP + (index_point - first_subj_on_screen), 8);
						putchar ('+');
					} else {
						draw_subject_arrow();
					}
					info_message(txt_thread_marked_as_unread);
					thread_marked_unread = FALSE;
				}
				break;

			default:
			    info_message(txt_bad_command);
		}
	}

group_done:
	fix_new_highest (sav_groupnum);
	update_newsrc (group, my_group[sav_groupnum]);
	clear_note_area ();

	if (index_point == -2)
		tin_done(0);
}


/*
 *  Correct highest[] for the group selection page display since
 *  new articles may have been read or marked unread
 */

void fix_new_highest (groupnum)
	int groupnum;
{
	register int i;
	int sum = 0;

	for (i = 0; i < top; i++) {
		if (arts[i].unread) {
			sum++;
		}
	}
	
	unread[groupnum] = sum;
}


void show_group_page (group)
	char *group;
{
	char new_resps[8];
	char resps[8];
	char from[LEN+1];
	char subject[LEN+1];
	int i, j;
	int n;
	int respnum;

#ifdef SIGTSTP
	signal(SIGTSTP, group_susp);
#endif

#ifdef USE_CLEARSCREEN
	ClearScreen();
#else
	MoveCursor(0, 0);		/* top left corner */
	CleartoEOLN ();
#endif

	printf("%s\r\n", nice_time());	/* time in upper left */

#ifndef USE_CLEARSCREEN
	MoveCursor(1, 0);
	CleartoEOLN ();
	MoveCursor(2, 0);
	CleartoEOLN ();
#endif

	StartInverse();
	center_line(1, group);
	EndInverse();

	MoveCursor(0, (COLS - (int) strlen (txt_type_h_for_help))+1);	/* upper right */
	if (mail_check()) {			/* you have mail message in */
		printf(txt_you_have_mail);
	} else {
		printf(txt_type_h_for_help);
	}

	MoveCursor(1, (COLS - (int) strlen (txt_type_h_for_help))+1);	/* in upper middle */
	if (kill_articles) {		/* display KILL on screen */
		printf("KILL ON   ");
	} else {
		printf("          ");
	}
	if (post_process) {		/* display POST on screen */
		printf("POST ON");
	} else {
		printf("       ");
	}
		
	MoveCursor(INDEX_TOP, 0);

	first_subj_on_screen = (index_point / NOTESLINES) * NOTESLINES;
	if (first_subj_on_screen < 0)
		first_subj_on_screen = 0;

	last_subj_on_screen = first_subj_on_screen + NOTESLINES;
	if (last_subj_on_screen >= top_base) {
		last_subj_on_screen = top_base;
		first_subj_on_screen = top_base - NOTESLINES;

		if (first_subj_on_screen < 0)
			first_subj_on_screen = 0;
	}

	for (j=0, i = first_subj_on_screen; i < last_subj_on_screen; i++, j++) {
		respnum = base[i];

		if (arts[respnum].tagged) {
			sprintf (new_resps, "%2d", arts[respnum].tagged);
		} else if (new_responses(i)) {
			strcpy (new_resps, " +");
		} else {
			strcpy (new_resps, "  ");
		}

		n = nresp(i);
		if (n)
			sprintf (resps, "%4d", n);
		else
			strcpy (resps, "    ");

		if (show_author) {
			my_strncpy (subject, arts[respnum].subject, max_subj);
			my_strncpy (from, arts[respnum].from, max_from);

			sprintf (screen[j].col, "  %4d %s  %-*s%s %-*s\r\n",
			       i+1, new_resps, max_subj, subject, resps, max_from, from);
		} else {
			sprintf(screen[j].col, "  %4d %s  %-*s %s\r\n",
			       i+1, new_resps, max_subj+max_from, arts[respnum].subject, resps);
		}
		printf("%s", screen[j].col);
	}

#ifndef USE_CLEARSCREEN
	CleartoEOS ();
#endif

	if (top_base <= 0)
		info_message(txt_no_arts);
	else if (last_subj_on_screen == top_base)
		info_message(txt_end_of_arts);

	if (top_base > 0)
		draw_subject_arrow();
}

void draw_subject_arrow()
{
	draw_arrow (INDEX_TOP + (index_point-first_subj_on_screen));
}

void erase_subject_arrow()
{
	erase_arrow (INDEX_TOP + (index_point-first_subj_on_screen));
}


int prompt_subject_num (ch, group)
	char ch;
	char *group;
{
	int num;

	clear_message();

	if ((num = parse_num(ch, txt_read_art)) == -1) {
		clear_message();
		return FALSE;
	}
	num--;		/* index from 0 (internal) vs. 1 (user) */

	if (num >= top_base)
		num = top_base - 1;

	if (num >= first_subj_on_screen
	&&  num < last_subj_on_screen) {
		erase_subject_arrow();
		index_point = num;
		draw_subject_arrow();
	} else {
#ifndef USE_CLEARSCREEN
		erase_subject_arrow();
#endif		
		index_point = num;
		show_group_page(group);
	}
	return TRUE;
}

/*
 *  Return the number of unread articles there are within a thread
 */

int new_responses (thread)
	int thread;
{
	int i;
	int sum = 0;

	for (i = base[thread]; i >= 0; i = arts[i].thread)
			if (arts[i].unread)
				sum++;
	
	return sum;
}


void clear_note_area ()
{
#ifndef USE_CLEARSCREEN
	MoveCursor (3, 0);		/* top left corner */
	CleartoEOS ();
#endif
}

/*
 *  Post an original article (not a followup)
 */

int post_base (group)
	char *group;
{
	FILE *fp;
	char nam[LEN+1];
	char ch;
	char ch_default = 'p';
	char subj[LEN+1];
	char buf[LEN+1];

	if (!parse_string(txt_post_subject, subj))
		return FALSE;
	if (subj[0] == '\0')
		return FALSE;

	setuid(real_uid);
	setgid(real_gid);

	sprintf(nam, "%s/.article", homedir);
	if ((fp = fopen(nam, "w")) == NULL) {
		error_message (txt_cannot_open, nam);
		setuid (tin_uid);
		setgid (tin_gid);
		return FALSE;
	}
	chmod(nam, 0600);

	fprintf(fp, "Subject: %s\n", subj);
	fprintf(fp, "Newsgroups: %s\n", group);
	fprintf(fp, "Distribution: \n");
	if (*my_org)
		fprintf(fp, "Organization: %s\n", my_org);
	if (*reply_to)
		fprintf(fp, "Reply-To: %s\n", reply_to);
	fprintf(fp, "\n");

	add_signature (fp, FALSE);
	fclose(fp);

	ch = 'e';
	while (1) {
		switch (ch) {
		case 'e':
			start_line_offset = 5;
			invoke_editor(nam);
			break;

		case 'a':
			clear_message ();
			setuid(tin_uid);
			setgid(tin_gid);
			return FALSE;

		case 'p':
			wait_message (txt_posting);
			sprintf (buf, "%s/inews -h < %s", LIBDIR, nam);
			if (invoke_cmd (buf)) {
				info_message (txt_art_posted);
				goto post_base_done;
			} else {
				info_message (txt_art_rejected);
				break;
			}
		}

		do {
			sprintf (msg, "%s%c", txt_abort_edit_post, ch_default);
			wait_message (msg);
			MoveCursor(LINES, (int) strlen (txt_abort_edit_post));
			if ((ch = ReadCh()) == CR)
				ch = ch_default;
		} while (ch != 'a' && ch != 'e' && ch != 'p');
	}

post_base_done:
	setuid (tin_uid);
	setgid (tin_gid);

	update_art_posted_file (group, subj);
	
	return TRUE;
}


int find_new_pos (old_top, old_artnum, cur_pos)
	int old_top;
	long old_artnum;
	int cur_pos;
{
	int pos;
	
	if (top != old_top) {
		if ((pos = valid_artnum (old_artnum)) >= 0) {
			if ((pos = which_base (pos)) >= 0) {
				return pos;
			} else {
				return top_base - 1;
			}
		} else {
			return top_base - 1;
		}
	}

	return cur_pos;
}

