/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : kill.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 16-08-91
 *  Release   : 1.0
 *  Notes     : kill articles
 *  Copyright : (c) Copyright 1991 by Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char index_file[LEN+1];

struct kill_t *killf;
int kill_num = 0;
int max_kill;

/*
 *  read_kill_file - read ~/.tin/kill file contents into kill array
 */

int read_kill_file ()
{
	char buf[LEN+1];
	FILE *fp;

	free_kill_array ();
	
	if ((fp = fopen (killfile, "r")) != NULL) {
		kill_num=0;
		while (fgets (buf, LEN, fp) != NULL) {
			if (buf[0] != '#') {
				if (kill_num == max_kill-1) {
					expand_kill ();
				}
				killf[kill_num].kill_type = atoi (buf);
				if (fgets (buf, LEN, fp) != NULL) {
					killf[kill_num].kill_group = atol (buf);
				} else {	
					goto corrupt_killfile;
				}	
				switch (killf[kill_num].kill_type) {
					case KILL_SUBJ:
						if (fgets (buf, LEN, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_subj = str_dup (buf);
						}
						break;
					case KILL_FROM:
						if (fgets (buf, LEN, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_from = str_dup (buf);
						}
						break;
					case KILL_BOTH:
						if (fgets (buf, LEN, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_subj = str_dup (buf);
						}
						if (fgets (buf, LEN, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_from = str_dup (buf);
						}
						break;
					default:
						goto corrupt_killfile;
				}
				kill_num++;
			}
		}
		fclose (fp);
		return TRUE;		
	} else {
		return FALSE;
	}

corrupt_killfile:
	fclose (fp);
	killf[kill_num].kill_type = 0;
	error_message ("corrupt kill file %s", killfile);
	return FALSE;
}

/*
 *  write_kill_file - write kill strings to ~/.tin/kill
 */

void write_kill_file ()
{
	FILE *fp;
	int i;
	
	if (kill_num && (fp = fopen (killfile, "w")) != NULL) {
		wait_message (txt_saving);
		fprintf (fp, "# 1st line  1=(Subject: only)  2=(From: only)  3=(Subject: & From:)\n");
		fprintf (fp, "# 2nd line  0=(kill on all newsgroups)  >0=(kill on specific newsgroup)\n");
		for (i=0 ; i < kill_num ; i++) {
			if (killf[i].kill_type && (killf[i].kill_subj || killf[i].kill_from)) {
				fprintf (fp, "#\n# kill description %d\n", i+1);
				fprintf (fp, "%d\n", killf[i].kill_type);
				fprintf (fp, "%ld\n", killf[i].kill_group);
				switch (killf[i].kill_type) {
					case KILL_SUBJ:
						fprintf (fp, "%s\n", killf[i].kill_subj);
						break;
					case KILL_FROM:
						fprintf (fp, "%s\n", killf[i].kill_from);
						break;
					case KILL_BOTH:
						fprintf (fp, "%s\n", killf[i].kill_subj);
						fprintf (fp, "%s\n", killf[i].kill_from);
						break;
				}
			}
		}
		fclose (fp);
		chmod (killfile, 0600);
	}
}

/*
 *  options menu so that the user can dynamically change parameters
 */
 
int kill_art_menu (group_name, index)
	char *group_name;
	int index;
{
	char text[LEN+1];
	char kill_from[LEN+1];
	char kill_subj[LEN+1];
	char kill_group[LEN+1];
	char ch_default = 's';
	char *str;	
	int ch;
	int counter;
	int kill_from_ok = FALSE;
	int kill_subj_ok = FALSE;
	int kill_every_group;
	
	sprintf (kill_group, "%s only", group_name);
	my_strncpy (text, arts[index].subject, 45);
	sprintf (kill_subj, txt_kill_subject, text);
	my_strncpy (text, arts[index].from, 45);
	sprintf (kill_from, txt_kill_from, text);
	text[0] = '\0';
	
	ClearScreen ();
	printf("%s\r\n", nice_time());	/* time in upper left */

	StartInverse ();
	center_line (1, txt_kill_menu);
	EndInverse ();
	
	MoveCursor(INDEX_TOP, 0);
	printf ("%s\r\n\r\n", txt_kill_text);
	printf ("%s\r\n\r\n\r\n", txt_kill_text_type);
	printf ("%s\r\n\r\n", kill_subj);
	printf ("%s\r\n\r\n\r\n", kill_from);
	printf ("%s%s", txt_kill_group, kill_group);
	fflush(stdout);

	show_menu_help (txt_help_kill_text);
	parse_menu_string (INDEX_TOP, strlen (txt_kill_text), text);

	if (text[0]) {
		show_menu_help (txt_help_kill_text_type);
		counter = 1;
		MoveCursor (INDEX_TOP+2, strlen (txt_kill_text_type));
		str = "Subject: line only    ";
		printf ("%s", str);
		fflush(stdout);
		do {
			MoveCursor (INDEX_TOP+2, strlen (txt_kill_text_type));
			if ((ch	= ReadCh()) == ' ') {
				counter++;
				if (counter == KILL_BOTH+1) {
					counter = KILL_SUBJ;
				}
				switch (counter) {
					case KILL_SUBJ:
						str = "Subject: line only    ";
						break;
					case KILL_FROM:
						str = "From: line only       ";
						break;
					case KILL_BOTH:
						str = "Subject: & From: lines";
						break;
				}
				printf ("%s", str);
				fflush(stdout);
			}
		} while (ch != CR);
	}

	if (! text[0]) {
		show_menu_help (txt_help_kill_subject);
		kill_subj_ok = prompt_yn (INDEX_TOP+5, kill_subj, 'y');

		show_menu_help (txt_help_kill_from);
		kill_from_ok = prompt_yn (INDEX_TOP+7, kill_from, 'n');
	}

	if (text[0] || kill_subj_ok || kill_from_ok) {
		show_menu_help (txt_help_kill_group);
		kill_every_group = FALSE;
		MoveCursor (INDEX_TOP+10, strlen (txt_kill_group));
		str = kill_group;
		printf ("%s", str);
		fflush(stdout);
		do {
			MoveCursor (INDEX_TOP+10, strlen (txt_kill_group));
			if ((ch	= ReadCh()) == ' ') {
				kill_every_group = !kill_every_group;
				if (kill_every_group) {
					str = "All groups";
				} else {
					str = kill_group;
				}
				CleartoEOLN (); 
				printf ("%s", str);
				fflush(stdout);
			}
		} while (ch != CR);
	}

	while (1) {
		do {
			sprintf (msg, "%s%c", txt_abort_edit_save_killfile, ch_default);
			wait_message (msg);
			MoveCursor(LINES, strlen (txt_abort_edit_save_killfile));
			if ((ch = ReadCh()) == CR)
				ch = ch_default;
		} while (ch != 'a' && ch != 'e' && ch != 's');
		switch (ch) {
			case 'e':
				start_line_offset = 2;
				invoke_editor (killfile);
				untag_all_articles ();
				read_kill_file ();
				reload_index_file (group_name, FALSE);
				return TRUE;

			case 'a':
				return FALSE;
			
			case 's':
				if (kill_num > max_kill-1) {
					expand_kill ();
				}
				if (text[0]) {
					switch (counter) {
						case KILL_SUBJ:
							killf[kill_num].kill_subj = str_dup (text);
							break;
						case KILL_FROM:
							killf[kill_num].kill_from = str_dup (text);
							break;
						case KILL_BOTH:
							killf[kill_num].kill_subj = str_dup (text);
							killf[kill_num].kill_from = str_dup (text);
							break;
					}
					killf[kill_num].kill_type = counter;
					if (kill_every_group) {
						killf[kill_num].kill_group= 0L;
					} else {
						killf[kill_num].kill_group= hash_s (group_name);
					}
					kill_num++;
				} else {
					if (kill_subj_ok) {
						killf[kill_num].kill_type = KILL_SUBJ;
						killf[kill_num].kill_subj = str_dup (arts[index].subject);
					}
					if (kill_from_ok) {
						killf[kill_num].kill_type |= KILL_FROM;
						killf[kill_num].kill_from = str_dup (arts[index].from);
					}
					if (killf[kill_num].kill_type) {		
						if (kill_every_group) {
							killf[kill_num].kill_group= 0L;
						} else {
							killf[kill_num].kill_group= hash_s (group_name);
						}
						kill_num++;
					}
				}
				write_kill_file ();
				return TRUE;
		}	
	}
}


int untag_all_articles ()
{
	int untagged = FALSE;
	register int i;

	for (i=0 ; i < top ; i++) {
		if (arts[i].tagged) {
			arts[i].tagged = FALSE;
			untagged = TRUE;
		}
	}
	num_of_tagged_files = 0;

	return (untagged);
}


int kill_any_articles (group)
	char *group;
{
	int killed = FALSE;
	int run_ok = FALSE;
	long group_hash;
	register int i, j;

	if (! kill_articles) {
		return killed;
	}

	if (kill_num) {
		group_hash = hash_s (group);
		for (i=0 ; i < kill_num ; i++) {
				if (killf[i].kill_group == 0L ||
					killf[i].kill_group == group_hash) {
					run_ok = TRUE;	
				}
		}
		if (! run_ok) {
			return (killed);
		}
		if (debug && ! update) {
			wait_message ("Killing articles...");
		}
		for (i=0 ; i < top ; i++) {
			for (j=0 ; j < kill_num && ! arts[i].tagged ; j++) {
				if (killf[j].kill_group == 0L ||
					killf[j].kill_group == group_hash) {
					switch (killf[j].kill_type) {
						case KILL_SUBJ:
							if (str_str (arts[i].subject, killf[j].kill_subj) != 0) {
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
						case KILL_FROM:
							if (str_str (arts[i].from, killf[j].kill_from) != 0) {
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
						case KILL_BOTH:
							if (str_str (arts[i].subject, killf[j].kill_subj) != 0) {
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							if (str_str (arts[i].from, killf[j].kill_from) != 0) {
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
					}
				}
			}
		}
	}
	return (killed);
}
