/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : main.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 20-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

char *version = "v1.00";
char cvers[LEN+1];
char nntp_server[LEN+1];

int max_subj = 0;
int max_from = 0;
int max_active;
int group_hash[TABLE_SIZE];			/* group name --> active[] */
int *my_group;						/* .newsrc --> active[] */
int *unread;						/* highest art read in group */
int num_active;						/* one past top of active */
int local_top;						/* one past top of my_group */
int catchup = FALSE;				/* mark all arts read in all subscribed groups */
int update = FALSE;					/* update index files only mode */
int verbose = FALSE;				/* update index files only mode */
int start_line_offset;				/* used by invoke_editor for line no. */
int read_news_via_nntp = FALSE;		/* read news locally or via NNTP */
int max_art;
int top = 0;
int top_base;
int tin_uid;
int tin_gid;
int real_uid;
int real_gid;
int local_index;					/* do private indexing? */

long *base;

struct group_ent *active;			/* active file */
struct header *arts;


#ifdef SIGTSTP
void main_susp (sig)
	int sig;
{
	Raw(FALSE);
	putchar('\n');
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask(sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
	kill(0, SIGTSTP);

	signal(SIGTSTP, main_susp);
	mail_setup();
	if (! update) {
		Raw(TRUE);
	}
}
#endif


void signal_handler (sig)
	int sig;
{
	if (sig == SIGINT) {
		signal (SIGINT, signal_handler);
	} else {
		free_all_arrays ();
		Raw(FALSE);
		printf("\n%s: signal handler caught signal %d\n", progname,sig);
		exit(1);
	}
}


void main (argc, argv)
	int argc;	
	char *argv[];
{
	extern int optind, opterr;
	extern char *optarg;
	extern char author_search_string[LEN+1];
	extern char group_search_string[LEN+1];
	extern char subject_search_string[LEN+1];
	int c, i;
	struct stat st;
	
	signal(SIGINT, signal_handler);		/* ctrl-C */
	signal(SIGQUIT, signal_handler);	/* ctrl-\ */
	signal(SIGILL, signal_handler);
	signal(SIGBUS, signal_handler);
	signal(SIGSEGV, signal_handler);
	
	signal(SIGPIPE, SIG_IGN);
#ifdef SIGTSTP
	signal(SIGTSTP, main_susp);
#endif

	debug = FALSE;

	strcpy (progname, argv[0]);

	nntp_server[0] = '\0';
	group_search_string[0] = '\0';
	author_search_string[0] = '\0';
	subject_search_string[0] = '\0';

	hash_init ();
	for (i = 0; i < TABLE_SIZE; i++) {
		group_hash[i] = -1;
	}

	tin_uid = geteuid();
	tin_gid = getegid();
	real_uid = getuid();
	real_gid = getgid();

	init_selfinfo ();	/* set up char *'s: homedir, newsrc, etc. */
	init_alloc ();		/* allocate initial array sizes */

	if (tin_uid == real_uid) {	/* run out of someone's account */
		local_index = TRUE;	/* index in their home directory */
		mkdir(indexdir, 0755);
	} else {			/* we're setuid, index in /usr/spool/news */
		local_index = FALSE;
	}

	while ((c = getopt (argc, argv, "cd:Df:hm:M:p:rS:uv")) != EOF) {
		switch(c) {
			case 'c':
				catchup = TRUE;
				update = TRUE;
				break;
				
			case 'd':
				my_strncpy (savedir, optarg, LEN);
				if (stat (savedir,&st) == -1) {
					mkdir (savedir, 0755);
				}
				break;

			case 'D':		/* debug mode */
				debug = TRUE;
				break;

			case 'f':
				my_strncpy (newsrc, optarg, LEN);
				break;

			case 'm':
				my_strncpy (mailer, optarg, LEN);
				break;

			case 'M':
				my_strncpy (maildir, optarg, LEN);
				if (stat (maildir,&st) == -1) {
					mkdir (maildir, 0755);
				}
				break;

			case 'p':
				my_strncpy (printer, optarg, LEN);
				default_printer = FALSE;
				break;

			case 'r':	/* read news remotely from default NNTP server */
#ifdef USE_NNTP			
				read_news_via_nntp = TRUE;
#else
				printf ("Option not enabled. Recompile %s with -DUSE_NNTP.\n", progname);
				exit (1);
#endif
				break;

			case 'S':
				my_strncpy (spooldir, optarg, LEN);
				break;

			case 'u':
				update = TRUE;
				break;

			case 'v':
				verbose = TRUE;
				update = TRUE;
				break;

			case 'h':
			case '?':
			default:
				usage (progname);
				exit (1);
		}
	}

	sprintf (page_header, "%s %s%s", progname, version, is_remote ()); 	
	sprintf (cvers, "%s (c) Copyright 1991 Iain Lea.", page_header);

	if (! update) {
		printf ("%s\n", cvers);
		fflush (stdout);
	}

	nntp_startup ();	/* connect server if we are using nntp */
	read_active ();		/* load the active file into active[] */

	if (optind < argc) {
		while (optind < argc) {
			if (add_group (argv[optind], TRUE) < 0) {
				fprintf (stderr, txt_not_in_active_file, argv[optind]);
				fprintf (stderr, "\n");
			}
			optind++;
		}
	} else {
		read_newsrc(TRUE);
	}

	if (InitScreen () == FALSE) {
		fprintf(stderr,txt_screen_init_failed);
		exit(1);
	}

	ScreenSize (&LINES, &COLS);
	Raw(TRUE);

#ifdef TIOCGWINSZ
	{
		struct winsize win;

		if (ioctl (0, TIOCGWINSZ, &win) == 0) {
			if (win.ws_row != 0)
				LINES = win.ws_row - 1;
			if (win.ws_col != 0)
				COLS = win.ws_col;
		}
	}
#endif

	max_subj = (COLS/2) - 2;
	max_from = (COLS-max_subj) - 17;

	if (kill_articles) {			/* read in users kill file */
		read_kill_file ();
	}
	
	if (update) {			/* index file updater only */
		Raw (FALSE);
		do_update ();
		exit (0);
	}

	init_screen_array (TRUE);		/* allocate screen array */
	
	mail_setup ();		/* record mailbox size for "you have mail" */

#ifndef USE_CLEARSCREEN
	ClearScreen();
#endif

	selection_index ();

	tin_done (0);

	exit (0);
}

/*
 * usage
 */

void usage (progname)
	char *progname;
{
    fprintf(stderr, "%s %s - threaded usenet newsreader. Copyright 1991 Iain Lea.\n\n", progname, version);
    fprintf(stderr, "Usage: %s [options] [newsgroups]\n",progname);
    fprintf(stderr, "  -c       mark all articles as read in subscribed newsgroups\n");
    fprintf(stderr, "  -d dir   save articles to directory [default=%s]\n",savedir);
    fprintf(stderr, "  -f file  subscribed newsgroups file [default=%s/.newsrc]\n", homedir);
    fprintf(stderr, "  -h       help\n");
    fprintf(stderr, "  -m file  mail program [default=%s]\n",DEFAULT_MAILER);
    fprintf(stderr, "  -M dir   mailbox directory [default=%s]\n",maildir);
    fprintf(stderr, "  -p file  print program with options [default=%s]\n",DEFAULT_PRINTER);
    fprintf(stderr, "  -r       read news remotely from default NNTP server\n",DEFAULT_PRINTER);
    fprintf(stderr, "  -S dir   spool directory [default=%s]\n",SPOOLDIR);
    fprintf(stderr, "  -u       update index files only\n");
    fprintf(stderr, "  -v       update index files only (verbose)\n");
}
