/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : misc.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 20-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"


char rcdir[LEN+1];
char rcfile[LEN+1];
char indexdir[LEN+1];
char killfile[LEN+1];
char postfile[LEN+1];

char active_file[LEN+1];
char homedir[LEN+1];
char userid[LEN+1];
char delgroups[LEN+1];
char newsrc[LEN+1];
char newnewsrc[LEN+1];
char add_addr[LEN+1];		/* address to add to rR reply to author with mail */
char reply_to[LEN+1];		/* reply-to address */
char my_org[LEN+1];			/* organization */
char sig[LEN+1];
char signature[LEN+1];
char killsubj[LEN+1];		/* contains Subject:'s not to be shown */
char killfrom[LEN+1];		/* conatins From:'s not to be shown */
char page_header[LEN+1];	/* page header of pgm name and version */
char savedir[LEN+1];		/* directory to save articles to */
char spooldir[LEN+1];		/* directory where new is */
char mailer[LEN+1];			/* mail program */
char maildir[LEN+1];		/* mailbox dir where = saves are stored */
char printer[LEN+1];		/* printer program */
char progname[LEN+1];		/* program name */
int sort_art_type;			/* sort arts[] array by subject,from or date field */
int save_separate;			/* save a each part of a thread to a separate file */
int save_archive_name;		/* save thread with name from Archive-name: field */
int default_printer;		/* set to false if user give a printer with -p switch */
int print_header;			/* print all of mail header or just Subject: & From lines */
int show_author;			/* show Subject & From or only Subject in group menu */
int draw_arrow_mark;		/* draw -> or highlighted bar */
int kill_articles;			/* kill articles matching kill file contents */
int post_process;			/* post process (ie. unshar/uudecode) saved article/thread */
int mark_saved_read;		/* mark saved article/thread as read */
int post_proc_type;			/* type of post processing to be performed */
int num_of_tagged_files;

/*
 * init_selfinfo
 *   Deterimines users home directory, userid, and a path
 *   for an rc file in the home directory
 */

void init_selfinfo()
{
	char nam[LEN+1];
	char *p;
	extern char *getenv();
	extern struct passwd *getpwuid();
	FILE *fp;
	struct passwd *myentry;
	struct stat sb;

	myentry = getpwuid (getuid());
	strcpy (userid, myentry->pw_name);
	strcpy (homedir, myentry->pw_dir);

	default_printer = TRUE;
	draw_arrow_mark = FALSE;
	kill_articles = FALSE;
	mark_saved_read = TRUE;
	num_of_tagged_files = 0;
	post_process = FALSE;
	post_proc_type = POST_PROC_SH;
	print_header = FALSE;
	save_separate = TRUE;
	save_archive_name = TRUE;
	show_author = TRUE;
	sort_art_type = SORT_BY_NONE;

	killsubj[0] = '\0';
	killfrom[0] = '\0';
	
	author_search_string[0] = '\0';
	group_search_string[0] = '\0';
	subject_search_string[0] = '\0';
	art_search_string[0] = '\0';
	default_regex_pattern[0] = '\0';
	
	sprintf (rcdir,  "%s/%s", homedir, RCDIR);
	sprintf (indexdir, "%s/%s", rcdir, INDEXDIR);
	sprintf (rcfile,   "%s/%s", rcdir, RCFILE);
	sprintf (indexdir, "%s/%s", rcdir, INDEXDIR);
	sprintf (killfile, "%s/%s", rcdir, KILLFILE);
	sprintf (postfile, "%s/%s", rcdir, POSTFILE);

	if (stat (rcdir, &sb) == -1) {
		mkdir (rcdir, 0755);
		if (stat (indexdir, &sb) == -1) {
			mkdir (indexdir, 0755);
		}
	}

	sprintf (signature, "%s/.signature", homedir);
	sprintf (sig, "%s/.Sig", homedir);
	sprintf (newsrc, "%s/.newsrc", homedir);
	sprintf (newnewsrc, "%s/.newnewsrc", homedir);
	sprintf (delgroups, "%s/.delgroups", homedir);
	sprintf (active_file, "%s/active", LIBDIR);
	sprintf (savedir, "%s/News",homedir);
	sprintf (maildir, "%s/%s",homedir,DEFAULT_MAILBOX);

	strcpy (mailer, DEFAULT_MAILER);
	strcpy (printer, DEFAULT_PRINTER);
	strcpy (spooldir, SPOOLDIR);

	read_rcfile ();

	if (stat(savedir,&sb) == -1) {
		mkdir(savedir, 0755);
	}
	if (stat(maildir,&sb) == -1) {
		mkdir(maildir, 0755);
	}
	
	if (stat(active_file, &sb) >= 0)
		goto got_active;

/*
 *  I hate forgetting to define LIBDIR correctly.  Guess a
 *  couple of likely places if it's not where LIBDIR says it is.
 */

	strcpy (active_file, "/usr/lib/news/active");
	if (stat(active_file, &sb) >= 0)
		goto got_active;

	strcpy(active_file, "/usr/local/lib/news/active");
	if (stat(active_file, &sb) >= 0)
		goto got_active;

	strcpy(active_file, "/usr/public/lib/news/active");
	if (stat(active_file, &sb) >= 0)
		goto got_active;

/*
 *  Oh well.  Revert to what LIBDIR says it is to produce a
 *  useful error message when read_active() fails later.
 */

	sprintf(active_file, "%s/active", LIBDIR);

got_active:

/*
 *  check enviroment for organization
 */
	my_org[0] = '\0';
	if ((p = getenv("ORGANIZATION")) != NULL) {
		my_strncpy (my_org, p, LEN);
		goto got_org;
	}

/*
 *  check ~/.org for organization
 */
	sprintf(nam, "%s/organization", rcdir);
	fp = fopen (nam, "r");

/*
 *  check LIBDIR/organization for system wide organization
 */
	if (fp == NULL) {
		sprintf(nam, "%s/organization", LIBDIR);
		fp = fopen(nam, "r");
	}

	if (fp == NULL) {
		sprintf(nam, "/usr/lib/news/organization");
		fp = fopen(nam, "r");
	}

	if (fp == NULL) {
		sprintf(nam, "/usr/local/lib/news/organization");
		fp = fopen(nam, "r");
	}

	if (fp == NULL) {
		sprintf(nam, "/usr/public/lib/news/organization");
		fp = fopen(nam, "r");
	}

	if (fp == NULL) {
		sprintf(nam, "/etc/organization");
		fp = fopen(nam, "r");
	}

	if (fp != NULL) {
		if (fgets(my_org, LEN, fp) != NULL) {
			for (p = my_org; *p && *p != '\n'; p++) ;
			*p = '\0';
		}
		fclose(fp);
	}

got_org:;

/*
 *  check enviroment for REPLY_TO
 */
	reply_to[0] = '\0';
	if ((p = getenv("REPLY_TO")) != NULL) {
		my_strncpy (reply_to, p, LEN);
		goto got_reply;
	}

	sprintf(nam, "%s/reply_to", rcdir);
	fp = fopen(nam, "r");
	
	if (fp != NULL) {
		if (fgets(reply_to, LEN, fp) != NULL) {
			for (p = reply_to; *p && *p != '\n'; p++) ;
			*p = '\0';
		}
		fclose(fp);
	}

got_reply:;

/*
 *  check enviroment for ADD_ADDR
 */
	add_addr[0] = '\0';
	if ((p = getenv("ADD_ADDR")) != NULL) {
		my_strncpy (add_addr, p, LEN);
		goto got_addr;
	}

	sprintf(nam, "%s/add_addr", rcdir);
	fp = fopen(nam, "r");
	
	if (fp != NULL) {
		if (fgets(add_addr, LEN, fp) != NULL) {
			for (p = add_addr; *p && *p != '\n'; p++) ;
			*p = '\0';
		}
		fclose(fp);
	}

got_addr:;

}


/*
 *  Which base note (an index into base[]) does a respnum
 *  (an index into arts[]) corresponsd to?
 *
 *  In other words, base[] points to an entry in arts[] which is
 *  the head of a thread, linked with arts[].thread.  For any q: arts[q],
 *  find i such that base[i]->arts[n]->arts[o]->...->arts[q]
 */

int which_base (n)
	int n;
{
	register int i, j;

	for (i = 0; i < top_base; i++) {
		for (j = base[i] ; j >= 0 ; j = arts[j].thread) {
			if (j == n) {
				return i;
			}
		}
	}

	error_message (txt_cannot_find_base_art, (char *) atoi (n));
	return -1;
}

/*
 *  Find how deep in a thread a response is.  Start counting at zero
 */

int which_resp (n)
	int n;
{
	int i, j;
	int num = 0;

	i = which_base (n);

	for (j = base[i]; j != -1; j = arts[j].thread)
		if (j == n)
			break;
		else
			num++;

	return num;
}

/*
 *  Given an index into base[], find the number of responses for
 *  that basenote
 */

int nresp (n)
	int n;
{
	int i;
	int oldi = -3;
	int sum = 0;

	assert(n < top_base);

	for (i = base[n]; i != -1; i = arts[i].thread) {
		assert(i != -2);
		assert(i != oldi);
		oldi = i;
		sum++;
	}

	return sum - 1;
}


void asfail(file, line, cond)
	char	*file;
	int	line;
	char	*cond;
{
	fprintf(stderr, "%s: assertion failure: %s (%d): %s\n",
							progname,file, line, cond);
	exit(1);
}


void copy_fp(a, b, prefix)
	FILE *a;
	FILE *b;
	char *prefix;
{
	char buf[8192];

	while (fgets(buf, 8192, a) != NULL)
		fprintf(b, "%s%s", prefix, buf);
}


char *get_val(env, def)
	char *env;		/* Environment variable we're looking for	*/
	char *def;		/* Default value if no environ value found	*/
{
	extern char *getenv();
	char *ptr;

	if ((ptr = getenv(env)) != NULL)
		return(ptr);
	else
		return(def);
}


int invoke_editor(nam)
	char *nam;
{
	char buf[LEN+1];
	static char editor[LEN+1];
	static int first = TRUE;

	if (first) {
		strcpy (editor, get_val("EDITOR", DEFAULT_EDITOR));
		first = FALSE;
	}

#ifdef DONT_USE_START_LINE
	sprintf(buf, "%s %s", editor, nam);
#else
	sprintf(buf, "%s +%d %s", editor, start_line_offset, nam);
#endif

	printf("%s", buf);
	return invoke_cmd(buf);
}


void shell_escape()
{
	char shell[LEN+1];
	char *p;

#ifdef SIGTSTP
	void (*susp)();
#endif

	if (!parse_string("!", shell))
		strcpy (shell, get_val("SHELL", DEFAULT_SHELL));

	for (p = shell; *p && (*p == ' ' || *p == '\t'); p++) ;

	if (!*p)
		strcpy (shell, get_val("SHELL", DEFAULT_SHELL));
	
	Raw(FALSE);

	setuid(real_uid);
	setgid(real_gid);

	fputs("\r\n", stdout);

#ifdef SIGTSTP
	susp = signal(SIGTSTP, SIG_DFL);
#endif

	system(p);

#ifdef SIGTSTP
	signal(SIGTSTP, susp);
#endif

	setuid(tin_uid);
	setgid(tin_gid);

	Raw(TRUE);

	mail_setup();
}


void tin_done (ret)
	int ret;
{
	free_all_arrays ();		/* deallocate all arrays */
	ClearScreen();
	Raw(FALSE);
	exit(ret);
}

/*
 *  Load the active file into active[]
 */

void read_active()
{
	FILE *fp;
	char *p, *q;
	char buf[LEN+1];
	long h;
	int i;

	num_active = 0;

	if ((fp = open_active_fp ()) == NULL) {
		fprintf (stderr, txt_cannot_open, active_file);
		fprintf (stderr, "\n");
		exit(1);
	}

	while (fgets(buf, LEN, fp) != NULL) {
		for (p = buf; *p && *p != ' '; p++) ;
		if (*p != ' ') {
			fprintf(stderr, txt_bad_active_file);
			continue;
		}
		*p++ = '\0';

		if (num_active >= max_active)
			expand_active();

		h = hash_groupname (buf);

		if (group_hash[h] == -1) {
			group_hash[h] = num_active;
		} else {				/* hash linked list chaining */
			for (i=group_hash[h]; active[i].next >= 0; i=active[i].next) {
				if (strcmp(active[i].name, buf) == 0) {
					goto read_active_continue;		/* kill dups */
				}
			}
			if (strcmp(active[i].name, buf) == 0)
				goto read_active_continue;
			active[i].next = num_active;
		}

		for (q = p; *q && *q != ' '; q++) ;
		if (*q != ' ') {
			fprintf(stderr, txt_bad_active_file);
			continue;
		}

		active[num_active].name = str_dup (buf);
		active[num_active].max = atol (p);
		active[num_active].min = atol (q);
		active[num_active].next = -1;	    /* hash chaining */
		active[num_active].flag = NOTGOT;   /* not in my_group[] yet */

		num_active++;

read_active_continue:;

	}

#ifdef SORT_ACTIVE_FILE		/* DOES NOT WORK - ERROR */
	if (num_active) {
		qsort (active, num_active, sizeof (struct group_ent), active_comp);
	}
#endif	

	debug_print_active ();

	fclose(fp);
}


#ifdef SORT_ACTIVE_FILE		/* DOES NOT WORK - ERROR */
int active_comp (s1, s2)
	struct group_ent *s1;
	struct group_ent *s2;
{
	/* s1->name less than s2->name */
	if (strcmp (s1->name, s2->name) < 0) {
		return -1;
	}
	/* s1->name greater than s2->name */
	if (strcmp (s1->name, s2->name) > 0) {
		return 1;
	}
	return 0;
}
#endif

void add_signature (fp, flag)
	FILE *fp;
	int flag;
{
	FILE *sigf;

	if ((sigf = fopen(signature, "r")) != NULL) {
		if (flag) {
			fprintf(fp, "\n--\n");
			copy_fp(sigf, fp, "");
		}
		fclose(sigf);
		return;
	}

	if ((sigf = fopen(sig, "r")) != NULL) {
		fprintf(fp, "\n--\n");
		copy_fp(sigf, fp, "");
		fclose(sigf);
	}
}


#ifdef USE_MKDIR
mkdir (path, mode)
	char *path;
	int mode;
{
	char buf[LEN+1];
	struct stat sb;

	sprintf(buf, "mkdir %s", path);
	if (stat (path, &sb) == -1) {
		system(buf);
		chmod(path, mode);
	}
}
#endif


long hash_groupname(buf)		/* hash group name for fast lookup later */
	char *buf;
{
	char *t = buf;
	unsigned long h;

	h = *t++;
	while (*t)
		h = ((h << 1) ^ *t++) % TABLE_SIZE;

	return h;
}


void rename_file (old_filename, new_filename)
	char *old_filename;
	char *new_filename;
{	
	unlink (new_filename);
	link (old_filename, new_filename);
	unlink (old_filename);
}


char *str_dup (str)
	char *str;
{
	char *dup = (char *) 0;

	assert (str != NULL);
	
	if (str) {
		dup = my_malloc (strlen (str)+1);
		strcpy (dup, str);
	}
	return dup;
}


int invoke_cmd(nam)
	char *nam;
{
	int ret;

#ifdef SIGTSTP
	void (*susp)();
#endif

	Raw(FALSE);
	setuid(real_uid);
	setgid(real_gid);

#ifdef SIGTSTP
	susp = signal(SIGTSTP, SIG_DFL);
#endif

	ret = system(nam);

#ifdef SIGTSTP
	signal(SIGTSTP, susp);
#endif

	setuid(tin_uid);
	setgid(tin_gid);
	Raw(TRUE);

	return ret == 0;
}
