/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : newsrc.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 10-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

/*
 *  Read $HOME/.newsrc into my_group[].  my_group[] ints point to
 *  active[] entries.  Sub_only determines whether we just read
 *  subscribed groups or all of them.
 */

void read_newsrc (sub_only)
	int sub_only;		/* TRUE=subscribed groups only, FALSE=all groups */
{
	FILE *fp;
	char *p;
	char buf[8192];
	char c;
	int i;

	local_top = 0;

	if ((fp = fopen(newsrc, "r")) == NULL) {  /* attempt to make a .newsrc */
		for (i = 0; i < num_active; i++) {
			if (local_top >= max_active)
				expand_active();
			my_group[local_top] = i;
			active[i].flag = 0;
			unread[local_top] = -1;
			local_top++;
		}
		write_newsrc ();
		return;
	}

	while (fgets(buf, 8192, fp) != NULL) {
		p = buf;
		while (*p && *p != '\n' && *p != ' ' && *p != ':' && *p != '!')
			p++;
		c = *p;
		*p++ = '\0';
		if (c == '!' && sub_only)
			continue;		/* unsubscribed */

		if ((i = add_group (buf, FALSE)) < 0) {
		    fprintf (stderr, txt_not_in_active_file, buf);
		    fprintf (stderr, "\n");
		    continue;
		}

		if (c != '!')		/* if we're subscribed to it */
			active[my_group[i]].flag |= SUBS;

		unread[i] = parse_unread (p, my_group[i]);
	}
	fclose(fp);
}


/*
 *  Write a new newsrc from my_group[] and active[]
 *  Used to a create a new .newsrc if there isn't one already, or when
 *  the newsrc is reset.
 */

void write_newsrc ()
{
	FILE *fp;
	int i;

	setuid(real_uid);	/* become the user to write in his */
	setgid(real_gid);	/* home directory */

	if ((fp = fopen(newsrc, "w")) == NULL)
		goto write_newsrc_done;

	for (i = 0; i < num_active; i++) {
		fprintf(fp, "%s: \n", active[i].name);
	}

	fclose(fp);

write_newsrc_done:
	setuid(tin_uid);
	setgid(tin_gid);
}

/*
 *  Load the sequencer rang lists and mark arts[] according to the
 *  .newsrc info for a particular group.  i.e.  rec.arts.comics: 1-94,97
 */

void read_newsrc_line(group)
	char *group;
{
	FILE *fp;
	char buf[8192];
	char *p;

	if ((fp = fopen(newsrc, "r")) == NULL)
		return;

	while (fgets(buf, 8192, fp) != NULL) {
		p = buf;
		while (*p && *p != '\n' && *p != ' ' && *p != ':' && *p != '!')
			p++;
		*p++ = '\0';
		if (strcmp(buf, group) != 0)
			continue;
		parse_seq(p);
		break;
	}

	fclose(fp);
}


/*
 *  For our current group, update the sequencer information in .newsrc
 */

void update_newsrc (group, groupnum)
	char *group;
	int groupnum;			/* index into active[] for this group */
{
	FILE *fp;
	FILE *newfp;
	char buf[8192];
	char *p;
	char c;

	setuid (real_uid);
	setgid (real_gid);

	if ((newfp = fopen (newnewsrc, "w")) == NULL)
		goto update_done;

	if ((fp = fopen (newsrc, "r")) != NULL) {
		while (fgets (buf, 8192, fp) != NULL) {
			for (p = buf; *p; p++)
				if (*p == '\n') {
					*p = '\0';
					break;
				}

			p = buf;
			while (*p && *p != ' ' && *p != ':' && *p != '!')
					p++;
			c = *p;
			if (c != '\0')
				*p++ = '\0';

			if (c != '!')
				c = ':';

			if (strcmp (buf, group) == 0) {
				fprintf (newfp, "%s%c ", buf, c);
				print_seq (newfp, groupnum);
				fprintf (newfp, "\n");
			} else
				fprintf (newfp, "%s%c%s\n", buf, c, p);
		}
		fclose (fp);
	}

	fclose (newfp);
	rename_file (newnewsrc, newsrc);

update_done:
	setuid (tin_uid);
	setgid (tin_gid);
}


/*
 *  Subscribe/unsubscribe to a group in .newsrc.  ch should either be
 *  '!' to unsubscribe or ':' to subscribe.  num is the group's index
 *  in active[].
 */

void subscribe (group, ch, num, out_seq)
	char *group;
	char ch;
	int num;
	int out_seq;				/* output sequencer info? */
{
	FILE *fp;
	FILE *newfp;
	char buf[8192];
	char *p;
	char c;
	int gotit = FALSE;

	if (ch == '!')
		active[num].flag &= ~SUBS;
	else
		active[num].flag |= SUBS;

	setuid (real_uid);
	setgid (real_gid);

	if ((newfp = fopen (newnewsrc, "w")) == NULL)
		goto subscribe_done;

	if ((fp = fopen (newsrc, "r")) != NULL) {
		while (fgets (buf, 8192, fp) != NULL) {
			for (p = buf; *p; p++)
				if (*p == '\n') {
					*p = '\0';
					break;
				}

			p = buf;
			while (*p && *p != ' ' && *p != ':' && *p != '!')
					p++;
			c = *p;
			if (c != '\0')
				*p++ = '\0';

			if (c != '!')
				c = ':';

			if (strcmp (buf, group) == 0) {
				fprintf (newfp, "%s%c%s\n", buf, ch, p);
				gotit = TRUE;
			} else
				fprintf (newfp, "%s%c%s\n", buf, c, p);
		}
		fclose (fp);
	}

	if (!gotit) {
		if (out_seq) {
			fprintf(newfp, "%s%c ", group, ch);
			print_seq(newfp, num);
			fprintf(newfp, "\n");
		} else
			fprintf(newfp, "%s%c\n", group, ch);
	}

	fclose (newfp);
	rename_file (newnewsrc, newsrc);

subscribe_done:
	setuid (tin_uid);
	setgid (tin_gid);
}


void reset_newsrc ()
{
	FILE *fp;
	FILE *newfp;
	char buf[8192];
	char *p;
	char c;
	int i;

	setuid(real_uid);
	setgid(real_gid);

	if ((newfp = fopen(newnewsrc, "w")) == NULL)
		goto update_done;

	if ((fp = fopen(newsrc, "r")) != NULL) {
		while (fgets(buf, 8192, fp) != NULL) {
			for (p = buf; *p && *p != '\n'; p++) ;
			*p = '\0';

			p = buf;
			while (*p && *p != ' ' && *p != ':' && *p != '!')
					p++;
			c = *p;
			if (c != '\0')
				*p++ = '\0';

			if (c != '!')
				c = ':';

			fprintf(newfp, "%s%c\n", buf, c);
		}
		fclose(fp);
	}

	fclose(newfp);
	rename_file (newnewsrc, newsrc);

update_done:
	setuid(tin_uid);
	setgid(tin_gid);

	for (i = 0; i < local_top; i++)
		unread[i] = -1;
}


void delete_group(group)
	char *group;
{
	FILE *fp;
	FILE *newfp;
	char buf[8192];
	char *p;
	char c;
	int gotit = FALSE;
	FILE *del;

	setuid(real_uid);
	setgid(real_gid);

	if ((newfp = fopen(newnewsrc, "w")) == NULL)
		goto del_done;

	if ((del = fopen(delgroups, "a+")) == NULL)
		goto del_done;

	if ((fp = fopen(newsrc, "r")) != NULL) {
		while (fgets(buf, 8192, fp) != NULL) {
			for (p = buf; *p && *p != '\n'; p++) ;
			*p = '\0';

			p = buf;
			while (*p && *p != ' ' && *p != ':' && *p != '!')
					p++;
			c = *p;
			if (c != '\0')
				*p++ = '\0';

			if (c != '!')
				c = ':';

			if (strcmp(buf, group) == 0) {
				fprintf(del, "%s%c%s\n", buf, c, p);
				gotit = TRUE;
			} else
				fprintf(newfp, "%s%c%s\n", buf, c, p);
		}
		fclose(fp);
	}

	fclose(newfp);

	if (!gotit)
		fprintf(del, "%s! \n", group);

	fclose(del);
	rename_file (newnewsrc, newsrc);

del_done:
	setuid(tin_uid);
	setgid(tin_gid);
}


int undel_group()
{
	FILE *del;
	FILE *newfp;
	FILE *fp;
	char buf[2][8192];
	char *p;
	int which = 0;
	long h;
	extern int cur_groupnum;
	int i, j;
	char c;

	setuid(real_uid);
	setgid(real_gid);

	if ((del = fopen(delgroups, "r")) == NULL) {
		setuid(tin_uid);
		setgid(tin_gid);
		return FALSE;
	}

	unlink(delgroups);
	
	if ((newfp = fopen(delgroups, "w")) == NULL) {
		setuid(tin_uid);
		setgid(tin_gid);
		return FALSE;
	}

	buf[0][0] = '\0';
	buf[1][0] = '\0';

	while (fgets(buf[which], 8192, del) != NULL) {
		which = !which;
		if (*buf[which])
			fputs(buf[which], newfp);
	}

	fclose(del);
	fclose(newfp);
	which = !which;

	if (!*buf[which]) {
		setuid(tin_uid);
		setgid(tin_gid);
		return FALSE;
	}

	for (p = buf[which]; *p && *p != '\n'; p++) ;
	*p = '\0';

	p = buf[which];
	while (*p && *p != ' ' && *p != ':' && *p != '!')
		p++;
	c = *p;
	if (c != '\0')
		*p++ = '\0';

	if (c != '!')
		c = ':';

	{			/* find the hash of the group name */
		char *t = buf[which];

		h = *t++;
		while (*t)
			h = (h * 64 + *t++) % TABLE_SIZE;
	}

	for (i = group_hash[h]; i >= 0; i = active[i].next) {
		if (strcmp(buf[which], active[i].name) == 0) {
			for (j = 0; j < local_top; j++)
				if (my_group[j] == i) {
					setuid(tin_uid);
					setgid(tin_gid);
					return j;
				}

			active[i].flag &= ~NOTGOT;   /* mark that we got it */
			if (c != '!')
				active[i].flag |= SUBS;

			if (local_top >= max_active)
				expand_active();
			local_top++;
			for (j = local_top; j > cur_groupnum; j--) {
				my_group[j] = my_group[j-1];
				unread[j] = unread[j-1];
			}
			my_group[cur_groupnum] = i;
			unread[cur_groupnum] = parse_unread(p, i);

			if ((fp = fopen(newsrc, "r")) == NULL) {
				setuid(tin_uid);
				setgid(tin_gid);
				return FALSE;
			}
			if ((newfp = fopen(newnewsrc, "w")) == NULL) {
				fclose(fp);
				setuid(tin_uid);
				setgid(tin_gid);
				return FALSE;
			}
			i = 0;
			while (fgets(buf[!which], 8192, fp) != NULL) {
				for (p = buf[!which]; *p && *p != '\n'; p++) ;
				*p = '\0';

				p = buf[!which];
				while (*p && *p!=' ' && *p != ':' && *p != '!')
					p++;
				c = *p;
				if (c != '\0')
					*p++ = '\0';

				if (c != '!')
					c = ':';

				while (i < cur_groupnum) {
					if (strcmp(buf[!which],
					  active[my_group[i]].name) == 0) {
						fprintf(newfp, "%s%c%s\n",
							buf[!which], c, p);
						goto foo_cont;
					}
					i++;
				}
				fprintf(newfp, "%s%c%s\n", buf[which], c, p);
				fprintf(newfp, "%s%c%s\n", buf[!which], c, p);
				break;
foo_cont:;
			}

			while (fgets(buf[!which], 8192, fp) != NULL)
				fputs(buf[!which], newfp);

			fclose(newfp);
			fclose(fp);
			rename_file (newnewsrc, newsrc);
			setuid(tin_uid);
			setgid(tin_gid);
			return TRUE;
		}
	}

	setuid(tin_uid);
	setgid(tin_gid);

	return FALSE;
}


void mark_group_read(group, groupnum)
	char *group;
	int groupnum;			/* index into active[] for this group */
{
	FILE *fp;
	FILE *newfp;
	char buf[8192];
	char *p;
	char c;

	if (active[groupnum].max < 2)
		return;

	setuid(real_uid);
	setgid(real_gid);

	if ((newfp = fopen(newnewsrc, "w")) == NULL)
		goto mark_group_read_done;

	if ((fp = fopen(newsrc, "r")) != NULL) {
		while (fgets(buf, 8192, fp) != NULL) {
			for (p = buf; *p; p++)
				if (*p == '\n') {
					*p = '\0';
					break;
				}

			p = buf;
			while (*p && *p != ' ' && *p != ':' && *p != '!')
					p++;
			c = *p;
			if (c != '\0')
				*p++ = '\0';

			if (c != '!')
				c = ':';

			if (strcmp(buf, group) == 0) {
				fprintf(newfp, "%s%c 1-%ld\n", buf, c,
						active[groupnum].max);
			} else
				fprintf(newfp, "%s%c%s\n", buf, c, p);
		}
		fclose(fp);
	}

	fclose(newfp);
	rename_file (newnewsrc, newsrc);

mark_group_read_done:
	setuid(tin_uid);
	setgid(tin_gid);
}


void parse_seq(s)
	char *s;
{
	long low, high;
	int i;

	while (*s) {
		while (*s && (*s < '0' || *s > '9'))
			s++;

		if (*s && *s >= '0' && *s <= '9') {
			low = atol(s);
			while (*s && *s >= '0' && *s <= '9')
				s++;
			if (*s == '-') {
				s++;
				high = atol(s);
				while (*s && *s >= '0' && *s <= '9')
					s++;
			}  else
				high = low;

			for (i = 0; i < top; i++)
				if (arts[i].artnum >= low && arts[i].artnum <= high)
					arts[i].unread = ART_READ;
		}
	}
}


int parse_unread (s, groupnum)
	char *s;
	int groupnum;			/* index for group in active[] */
{
	long low, high;
	long last_high;
	int sum = 0;
	int gotone = FALSE;
	int n;

/*
 *  Read the first range from the .newsrc sequencer information.  If the
 *  top of the first range is higher than what the active file claims is
 *  the bottom, use it as the new bottom instead
 */

	high = 0;
	if (*s) {
		while (*s && (*s < '0' || *s > '9'))
			s++;

		if (*s && *s >= '0' && *s <= '9') {
			low = atol(s);
			while (*s && *s >= '0' && *s <= '9')
				s++;
			if (*s == '-') {
				s++;
				high = atol(s);
				while (*s && *s >= '0' && *s <= '9')
					s++;
			}  else
				high = low;
			gotone = TRUE;
		}
	}

	if (high < active[groupnum].min)
		high = active[groupnum].min;

	while (*s) {
		last_high = high;

		while (*s && (*s < '0' || *s > '9'))
			s++;

		if (*s && *s >= '0' && *s <= '9') {
			low = atol(s);
			while (*s && *s >= '0' && *s <= '9')
				s++;
			if (*s == '-') {
				s++;
				high = atol(s);
				while (*s && *s >= '0' && *s <= '9')
					s++;
			}  else
				high = low;

			if (low > last_high)	/* otherwise seq out of order */
				sum += (low - last_high) - 1;
		}
	}

	if (gotone) {
		if (active[groupnum].max > high)
			sum += active[groupnum].max - high;
		return sum;
	}

	n = (int) (active[groupnum].max - active[groupnum].min);
	if (n < 2)
		return 0;

	return -1;
}


int get_line_unread(group, groupnum)
	char *group;
	int groupnum;				/* index for group in active[] */
{
	FILE *fp;
	char buf[8192];
	char *p;
	int ret = -1;

	if ((fp = fopen(newsrc, "r")) == NULL)
		return -1;

	while (fgets(buf, 8192, fp) != NULL) {
		p = buf;
		while (*p && *p != '\n' && *p != ' ' && *p != ':' && *p != '!')
			p++;
		*p++ = '\0';
		
		if (strcmp (buf, group) != 0)
			continue;
			
		ret = parse_unread (p, groupnum);
		break;
	}

	fclose(fp);
	return ret;
}


void print_seq (fp, groupnum)
	FILE *fp;
	int groupnum;			/* index into active[] for this group */
{
	int i;
	int flag = FALSE;

	if (top <= 0) {
		if (active[groupnum].min > 1) {
			fprintf (fp, "1-%ld", active[groupnum].min);
			fflush (fp);
		}
		return;
	}

	/*
	 *  sort into the same order as in the spool area for writing
	 *  read article numbers to ~/.newsrc
	 */
	qsort (arts, top, sizeof (struct header), artnum_comp);

	i = 0;
	if (arts[0].artnum > 1) {
		for (; i < top && !arts[i].unread; i++) ;
		if (i > 0)
			fprintf (fp, "1-%ld", arts[i-1].artnum);
		else
			fprintf (fp, "1-%ld", arts[0].artnum - 1);
		flag = TRUE;
	}

	for (; i < top; i++) {
		if (! arts[i].unread) {
			if (flag)
				fprintf(fp, ",");
			else
				flag = TRUE;
			fprintf (fp, "%ld", arts[i].artnum);
			if (i+1 < top && !arts[i+1].unread) {
				while (i+1 < top && ! arts[i+1].unread)
					i++;
				fprintf (fp, "-%ld", arts[i].artnum);
			}
		}
	}

	if (! flag && active[groupnum].min > 1)
		fprintf (fp, "1-%ld", active[groupnum].min);
	fflush (fp);

	/*
	 *  resort into required sort order
	 */
	switch (sort_art_type) {
		case SORT_BY_NONE:		/* already sorted above */
			break;
		case SORT_BY_SUBJ:
			qsort (arts, top, sizeof (struct header), subj_comp);
			break;
		case SORT_BY_FROM:
			qsort (arts, top, sizeof (struct header), from_comp);
			break;
		case SORT_BY_DATE:
			qsort (arts, top, sizeof (struct header), date_comp);
			break;
	}
}

/*
 *  rewrite .newsrc and position group at specifed position
 */

int pos_group_in_newsrc (group, pos)
	char *group;
	int pos;
{
	char sub[LEN+1];
	char unsub[LEN+1];
	char buf[LEN+1], *ptr;
	char newsgroup[LEN+1];
	extern int cur_groupnum;
	FILE *fp_in, *fp_out;
	FILE *fp_sub, *fp_unsub;
	int repositioned = FALSE;
	int subscribed_pos = 1;
	int group_len;

	if ((fp_in = fopen (newsrc, "r")) == NULL) {
		return FALSE;
	}
	if ((fp_out = fopen (newnewsrc, "w")) == NULL) {
		return FALSE;
	}

	sprintf (sub, "/tmp/.subrc%d", getpid ());
	sprintf (unsub, "/tmp/.unsubrc.%d", getpid ());

	if ((fp_sub = fopen (sub, "w")) == NULL) {
		return FALSE;
	}
	if ((fp_unsub = fopen (unsub, "w")) == NULL) {
		return FALSE;
	}

	/*
	 *  split newsrc into subscribed and unsubscribed to files
	 */
	group_len = strlen (group);

	while (fgets (buf, LEN, fp_in) != NULL) {
		if (strncmp (group, buf, group_len) == 0 && buf[group_len] == ':') {
			my_strncpy (newsgroup, buf, LEN);
		} else if (ptr = (char *) strchr (buf, ':')) {
			fprintf (fp_sub, "%s", buf);
		} else {
			fprintf (fp_unsub, "%s", buf);
		}
	}

	fclose (fp_in);
	fclose (fp_sub);
	fclose (fp_unsub);

	/*
	 *  write subscribed groups & position group to newnewsrc
	 */
	if ((fp_sub = fopen (sub, "r")) == NULL) {
		unlink (sub);
		return FALSE;
	}
	while (fgets (buf, LEN, fp_sub) != NULL) {
		if (pos == subscribed_pos) {
			fprintf (fp_out, "%s\n", newsgroup);
			repositioned = TRUE;
		}
		
		fprintf (fp_out, "%s", buf);

		subscribed_pos++;
	}
	if (! repositioned) {
		fprintf (fp_out, "%s", newsgroup);
		repositioned = TRUE;
	}
	
	fclose (fp_sub);
 	unlink (sub);

	/*
	 *  write unsubscribed groups to newnewsrc
	 */
	if ((fp_unsub = fopen (unsub, "r")) == NULL) {
		unlink (unsub);
		return FALSE;
	}
	while (fgets (buf, LEN, fp_unsub) != NULL) {
			fprintf (fp_out, "%s", buf);
	}

	fclose (fp_unsub);
	unlink (unsub);
	fclose (fp_out);

	if (repositioned) {
		cur_groupnum = pos;
		rename_file (newnewsrc, newsrc);
		return TRUE;
	} else {
		return FALSE;
	}
}
