/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : page.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 19-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

#define		NOTE_UNAVAIL	-1

extern int cur_groupnum;

char note_h_path[LEN+1];			/* Path:	*/
char note_h_date[LEN+1];			/* Date:	*/
char note_h_subj[LEN+1];			/* Subject:	*/
char note_h_from[LEN+1];			/* From:	*/
char note_h_org[LEN+1];				/* Organization: */
char note_h_newsgroups[LEN+1];		/* Newsgroups:	*/
char note_h_messageid[LEN+1];		/* Message-ID:	*/
char note_h_distrib[LEN+1];			/* Distribution: */
char note_h_followup[LEN+1];		/* Followup-To: */

char note_full_name[100];
char note_from_addr[100];
char *glob_page_group;

FILE *note_fp;				/* the body of the current article */

int	note_line;
int	note_page;					/* what page we're on */
int	note_end;					/* we're done showing this article */
int	rotate;						/* 0=normal, 13=rot13 decode */
int last_resp;		/* current & previous article for - command */
int this_resp;
int glob_respnum;

long	note_mark[MAX_PAGES];	/* ftells on beginnings of pages */
long note_size;					/* stat size in bytes of article */


#ifdef SIGTSTP
void page_susp (sig)
	int sig;
{

	Raw(FALSE);
	putchar('\n');
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask(sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
	kill(0, SIGTSTP);

	signal(SIGTSTP, page_susp);
	mail_setup();

	if (! update) {
		Raw(TRUE);
	}
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	redraw_page(glob_respnum, glob_page_group);
}
#endif


int show_page (respnum, group, group_path)
	int respnum;
	char *group;
	char *group_path;
{
	char ch;
	int i, n;
	int kill_state = NO_KILLING;
	int old_artnum;
	int old_sort_art_type = sort_art_type;
	int old_top;
	long art;

restart:

	glob_respnum = respnum;
	glob_page_group = group;

#ifdef SIGTSTP
	signal(SIGTSTP, page_susp);
#endif

	if (respnum != this_resp) {	   /* remember current & previous */
		last_resp = this_resp;	   /* articles for - command */
		this_resp = respnum;
	}

	rotate = 0;			/* normal mode, not rot13 */
	art = arts[respnum].artnum;
	arts[respnum].unread = ART_READ;	/* mark article as read */
	open_note(art, group_path);

	if (note_page == NOTE_UNAVAIL) {
		ClearScreen();
		printf(txt_art_unavailable, art);
		fflush(stdout);
	} else {
		show_note_page (respnum, group);
	}

	while (1) {
		ch = ReadCh();

		if (ch >= '0' && ch <= '9') {

			n = prompt_response(ch, respnum);
			if (n != -1) {
				respnum = n;
				goto restart;
			}

		} else switch (ch) {
			case 27:
				ch = ReadCh ();
				if (ch == '[' || ch == 'O')
					ch = ReadCh ();
				switch (ch) {
				case 'G':		/* ansi  PgDn */
				case 'U':		/* at386 PgDn */
					goto page_down;

				case 'I':		/* ansi  PgUp */
				case 'V':		/* at386 PgUp */
					goto page_up;

				case 'H':		/* at386 Home */
					goto begin_of_article;

				case 'F':		/* ansi End */
				case 'Y':		/* at386 End */
					goto end_of_article;
				}
				break;

			case '!':
				shell_escape();
				redraw_page(respnum, group);
				break;

			case '$':	/* goto end of article */
end_of_article:			
				if (show_last_page ()) {
					show_note_page(respnum, group);
				}
				break;

			case '-':	/* show last viewed article */
				if (last_resp < 0) {
					info_message(txt_no_last_message);
					break;
				}
				note_cleanup();
				respnum = last_resp;
				goto restart;

			case '|':	/* pipe article into command */
				pipe_article ();
				redraw_page (respnum, group);
				break;

			case '/':	/* search forwards in article */
				if (search_article (TRUE)) {
					show_note_page (respnum, group);
				}
				break;

			case ctrl('D'):
			case ' ': 	/* next page or response */
page_down:
				if (note_page == NOTE_UNAVAIL) {
					n = next_response (respnum);
					if (n == -1) {
						return (which_base (respnum));
					}
					respnum = n;
					goto restart;
				} else if (note_end) {
					note_cleanup ();
					n = next_response (respnum);
					if (n == -1) {
						return (which_base (respnum));
					}
					respnum = n;
					goto restart;
				} else
					show_note_page (respnum, group);
				break;

			case '\r':
			case '\n':	/* go to start of next thread */
				note_cleanup ();
				n = next_basenote (respnum);
				if (n == -1)
					return (which_base (respnum));

				respnum = n;
				goto restart;

			case '\t': 	/* next page or unread response */
				if (note_page == NOTE_UNAVAIL) {
					n = next_unread (next_response (respnum));
					if (n == -1)
						return (which_base (respnum));

					respnum = n;
					goto restart;

				} else if (note_end) {
					note_cleanup();
					n = next_unread(next_response(respnum));
					if (n == -1)
						return (which_base (respnum));

					respnum = n;
					goto restart;
				} else
					show_note_page(respnum, group);
				break;

			case ctrl('H'):	/* show article headers */
				if (note_page == NOTE_UNAVAIL) {
					n = next_response (respnum);
					if (n == -1)
						return (which_base (respnum));

					respnum = n;
					goto restart;
				} else {
					note_page = 0;
					note_end = FALSE;
					fseek(note_fp, 0L, 0);
					show_note_page(respnum, group);
				}
				break;

			case ctrl('K'):		/* kill article */
				if (kill_articles) {
					if (kill_art_menu (group, respnum)) {
						if (kill_any_articles (group)) {
							reload_index_file (group, TRUE);
						}
					}
					redraw_page(respnum, group);
				} else {
					info_message (txt_switch_on_kill_art_menu);
				}
				break;

			case ctrl('L'):	  /* redraw current page of article */
				redraw_page (respnum, group);
				break;

			case ctrl('R'):	  /* redraw beginning of article */
begin_of_article:			
				if (note_page == NOTE_UNAVAIL) {
					ClearScreen();
					printf(txt_art_unavailable,arts[respnum].artnum);
					fflush(stdout);
				} else {
					note_page = 0;
					note_end = FALSE;
					fseek(note_fp, note_mark[0], 0);
					show_note_page(respnum, group);
				}
				break;

			case ctrl('X'):
			case '%':
			case 'd':	/* toggle rot-13 mode */
				if (rotate)
					rotate = 0;
				else
					rotate = 13;
				redraw_page (respnum, group);
				break;

			case 'a':	/* author search forward */
			case 'A':	/* author search backward */
				i = (ch == 'a');
				n = search_author (respnum, i);
				if (n < 0)
					break;
				respnum = n;
				goto restart;
				break;

			case ctrl('U'):
			case 'b':		/* back a page */
page_up:
				if (note_page == NOTE_UNAVAIL) {
					note_cleanup();
					n = prev_response(respnum);
					if (n == -1)
						return( which_resp(respnum) );

					respnum = n;
					goto restart;

				} else {
					if (note_page <= 1) {
						info_message (txt_begin_of_art);
					} else {
						note_page -= 2;
						note_end = FALSE;
						fseek(note_fp, note_mark[note_page], 0);
						show_note_page(respnum, group);
					}
				}
				break;

			case 'B':	/* bug/gripe/comment mailed to author */
				mail_bug_report ();
				redraw_page (respnum, group);
				break;
				
			case 'c':	/* catchup--mark all articles as read */
			    if (prompt_yn (LINES, txt_mark_all_read, 'y')) {
					for (n = 0; n < top; n++) {
						arts[n].unread = ART_READ;
					}
					fix_new_highest(cur_groupnum);
					if (cur_groupnum + 1 < local_top) {
						cur_groupnum++;
					}
					note_cleanup();
					return -1;
			    }
				break;
				
			case 'f':	/* post a followup to this article */
				if (post_response (group, FALSE)) {
					update_newsrc (group, my_group[cur_groupnum]);
					n = which_base (respnum);
					note_cleanup ();
					index_group (group, group_path);
					read_newsrc_line (group);
					respnum = choose_resp (n, nresp(n));
					goto restart;
				} else
					redraw_page (respnum, group);
				break;

			case 'F':	/* post a followup to this article */
				if (post_response (group, TRUE)) {
					update_newsrc (group, my_group[cur_groupnum]);
					n = which_base (respnum);
					note_cleanup ();
					index_group (group, group_path);
					read_newsrc_line (group);
					respnum = choose_resp (n, nresp(n));
					goto restart;
				} else
					redraw_page (respnum, group);
				break;

			case 'h':	/* help overview */
				help_page_commands ();
				redraw_page (respnum, group);
				break;

			case 'H':	/* help in-depth */
				help_page_info ();
				redraw_page(respnum, group);
				break;

			case 'i':	/* return to index page */
return_to_index:
				note_cleanup ();
				if (kill_state == NO_KILLING &&
					sort_art_type != old_sort_art_type) {
					make_threads (TRUE);
					find_base ();
				}
				if (kill_state == KILLING) {
					old_top = top;
					old_artnum = arts[respnum].artnum;
					if (kill_articles) {
						kill_any_articles (group);
						reload_index_file (group, TRUE);	/* kill arts */
					} else {
						reload_index_file (group, FALSE);	/* unkill arts */
					}
					return find_new_pos (old_top, old_artnum, respnum);
				}
				return (which_base (respnum));

			case 'I':	/* toggle inverse video */
				inverse_okay = !inverse_okay;
				if (inverse_okay)
					info_message(txt_inverse_on);
				else
					info_message(txt_inverse_off);
				redraw_page (respnum, group);
				break;

			case 'k':
				if (note_page == NOTE_UNAVAIL) {
					n = next_unread(next_response(respnum));
					if (n == -1)
						return (which_base (respnum));

					respnum = n;
					goto restart;

				} else {
					note_cleanup();
					n = next_unread(next_response(respnum));
					if (n == -1)
						return (which_base (respnum));

					respnum = n;
					goto restart;
				}
				break;

			case 'K':	/* mark rest of thread as read */
				for (n = respnum; n >= 0; n = arts[n].thread)
					arts[n].unread = ART_READ;
				n = next_unread(next_response(respnum));
				if (n == -1)
					goto return_to_index;
				else {
					note_cleanup();
					respnum = n;
					goto restart;
				}
				break;

			case 'm':	/* mail article to somebody */
				mail_to_someone ();
				redraw_page (respnum, group);
				break;

			case 'M':	/* options menu */
				if (change_rcfile (group, FALSE) == KILLING) {
					kill_state = KILLING;
				} 
				redraw_page (respnum, group);
			    break;

			case 'n':	/* skip to next article */
				note_cleanup ();
				n = next_response (respnum);
				if (n == -1)
					return (which_base(respnum));

				respnum = n;
				goto restart;

			case 'N':	/* next unread article */
				n = next_unread(next_response(respnum));
				if (n == -1)
					info_message(txt_no_next_unread_art);
				else {
					note_cleanup();
					respnum = n;
					goto restart;
				}
				break;

			case 'o':	/* output article to printer */
				if (prompt_yn (LINES, txt_print_art_yn, 'y')) {
					print_article ();
				}
				break;

			case 'O':	/* output thread to printer */
				if (prompt_yn (LINES, txt_print_thread_yn, 'y')) {
					print_thread (respnum, group_path);
				}
				break;

			case 'p':	/* previous article */
				note_cleanup ();
				n = prev_response (respnum);
				if (n == -1)
					return (which_resp (respnum));

				respnum = n;
				goto restart;

			case 'P':	/* previous unread article */
				n = prev_unread(prev_response(respnum));
				if (n == -1)
				    info_message(txt_no_prev_unread_art);
				else {
					note_cleanup();
					respnum = n;
					goto restart;
				}
				break;

			case 'q':	/* quit */
				if (prompt_yn (LINES, txt_quit, 'y')) {
					return -2;
				}
				break;
	
			case 'r':	/* reply to author through mail */
				mail_to_author (FALSE);
				redraw_page (respnum, group);
				break;

			case 'R':	/* reply to author, copy text */
				mail_to_author (TRUE);
				redraw_page (respnum, group);
				break;

			case 's':
				save_art_to_file (respnum, 0, FALSE, NULL);
				if (post_process_files ()) {
					redraw_page (respnum, group);
				}
				free_save_array ();
				break;

			case 'S':
				save_thread_to_file (respnum, group_path);
				if (post_process_files ()) {
					redraw_page (respnum, group);
				}
				break;

			case 't':	/* return to group selection page */
				note_cleanup();
				if (kill_state == KILLING) {
					if (kill_articles) {
						kill_any_articles (group);
						reload_index_file (group, TRUE);	/* kill arts */
					} else {
						reload_index_file (group, FALSE);	/* unkill arts */
					}
				}
				update_newsrc (group, my_group[cur_groupnum]);
				fix_new_highest (cur_groupnum);
				return -1;

			case 'v':
				info_message (cvers);
				break;

			case 'w':	/* post a basenote */
				post_base (group);
				update_newsrc (group, my_group[cur_groupnum]);
				index_group (group, group_path);
				read_newsrc_line (group);
				redraw_page (respnum, group);
				break;

			case 'W':	/* display messages posted by user */
				if (user_posted_messages ()) {
					redraw_page(respnum, group);
				}
				break;

			case 'z':	/* mark article as unread (to return) */
				arts[respnum].unread = ART_WILL_RETURN;
				info_message(txt_art_marked_as_unread);
				break;

			default:
			    info_message(txt_bad_command);
		}
	}
}


void redraw_page(respnum, group)
	int respnum;
	char *group;
{

	if (note_page == NOTE_UNAVAIL) {
		ClearScreen();
		printf(txt_art_unavailable, arts[respnum].artnum);
		fflush(stdout);
	} else if (note_page > 0) {
		note_page--;
		fseek(note_fp, note_mark[note_page], 0);
		show_note_page(respnum, group);
	}
}


void show_note_page (respnum, group)
	int respnum;
	char *group;
{
	char buf[LEN+1];
	char buf2[LEN+50];
	int percent;
	char *p, *q;
	int i, j;
	int ctrl_L;		/* form feed character detected */

	ClearScreen();

	note_line = 1;

	if (note_page == 0)
		show_first_header (respnum, group);
	else
		show_cont_header (respnum);

	ctrl_L = FALSE;
	while (note_line < LINES) {
		if (fgets(buf, LEN, note_fp) == NULL) {
			note_end = TRUE;
			break;
		}

		buf[LEN-1] = '\0';
		if (rotate)
			for (p = buf, q = buf2;
					*p && *p != '\n' && q<&buf2[LEN]; p++) {
				if (*p == '\b' && q > buf2) {
					q--;
				} else if (*p == 12) {		/* ^L */
					*q++ = '^';
					*q++ = 'L';
					ctrl_L = TRUE;
				} else if (*p == '\t') {
					i = q - buf2;
					j = (i|7) + 1;

					while (i++ < j)
						*q++ = ' ';
				} else if (((*p) & 0x7F) < 32) {
					*q++ = '^';
					*q++ = ((*p) & 0x7F) + '@';
				} else if (*p >= 'A' && *p <= 'Z')
					*q++ = 'A' + (*p - 'A' + rotate) % 26;
				else if (*p >= 'a' && *p <= 'z')
					*q++ = 'a' + (*p - 'a' + rotate) % 26;
				else
					*q++ = *p;
			}
		else
			for (p = buf, q = buf2;
					*p && *p != '\n' && q<&buf2[LEN]; p++) {
				if (*p == '\b' && q > buf2) {
					q--;
				} else if (*p == 12) {		/* ^L */
					*q++ = '^';
					*q++ = 'L';
					ctrl_L = TRUE;
				} else if (*p == '\t') {
					i = q - buf2;
					j = (i|7) + 1;

					while (i++ < j)
						*q++ = ' ';
				} else if (((*p) & 0x7F) < 32) {
					*q++ = '^';
					*q++ = ((*p) & 0x7F) + '@';
				} else
					*q++ = *p;
			}

		*q = '\0';

		printf("%s\r\n", buf2);

		note_line += ((int) strlen (buf2) / COLS) + 1;

		if (ctrl_L) {
			break;
		}
	}

	note_mark[++note_page] = ftell(note_fp);

	StartInverse();	

	if (note_end) {
		MoveCursor(LINES, MORE_POS-5);
		if (arts[respnum].thread != -1)
			printf(txt_next_resp);
		else
			printf(txt_last_resp);
	} else {
		if (note_size > 0) {
		    percent = note_mark[note_page] * 100 / note_size;
		    sprintf (buf, "%s(%d%%) [%ld/%ld]", txt_more, percent, note_mark[note_page], note_size);
			MoveCursor (LINES, (COLS - (int) strlen (buf))-1);
			CleartoEOLN ();
			printf ("%s", buf);
		} else {
			MoveCursor(LINES, MORE_POS);
		    printf(txt_more);
		}
	}

	EndInverse();

	fflush(stdout);
}


void show_first_header (respnum, group)
	int respnum;
	char *group;
{
	int whichresp;
	int x_resp;
	char buf[LEN+1];
	char tmp[LEN+1];
	int pos, i;
	int n;

	whichresp = which_resp (respnum);
	x_resp = nresp (which_base (respnum));

	ClearScreen ();

	strcpy (buf, note_h_date);
	pos = (COLS - (int) strlen (group)) / 2;
	for (i = strlen(buf); i < pos; i++)
		buf[i] = ' ';
	buf[i] = '\0';

	strcat (buf, group);

	for (i = strlen(buf); i < RIGHT_POS; i++)
		buf[i] = ' ';
	buf[i] = '\0';

	printf (txt_note_x_of_n, buf, which_base (respnum) + 1, top_base);

	sprintf (buf, txt_art, arts[respnum].artnum);
	n = strlen (buf);
	fputs (buf, stdout);

	strcpy (buf, note_h_subj);
	buf[RIGHT_POS - 2 - n] = '\0';

	pos = ((COLS - (int) strlen (buf)) / 2) - 2;

	if (pos > n) {
		MoveCursor (1, pos);
	} else {
		MoveCursor (1, n);
	}

	StartInverse ();
	fputs (buf, stdout);
	EndInverse ();

	MoveCursor (1, RIGHT_POS);
	if (whichresp)
		printf (txt_resp_x_of_n, whichresp, x_resp);
	else {
		if (x_resp == 0)
			printf (txt_no_resp);
		else if (x_resp == 1)
			printf (txt_1_resp);
		else
			printf (txt_x_resp, x_resp);
	}

	if (*note_h_org)
		sprintf (tmp, txt_s_at_s, note_full_name, note_h_org);
	else
		strcpy (tmp, note_full_name);

	tmp[LEN] = '\0';

	sprintf (buf, "%s  ", note_from_addr);

	pos = COLS - 1 - (int) strlen(tmp);
	if ((int) strlen (buf) + (int) strlen (tmp) >= COLS - 1) {
		strncat (buf, tmp, COLS - 1 - (int) strlen(buf));
		buf[COLS - 1] = '\0';
	} else {
		for (i = strlen(buf); i < pos; i++)
			buf[i] = ' ';
		buf[i] = '\0';
		strcat (buf, tmp);
	}
	printf ("%s\r\n\r\n", buf);

	note_line += 4;
}


void show_cont_header (respnum)
	int respnum;
{
	int whichresp;
	int whichbase;
	char buf[LEN+1];

	whichresp = which_resp (respnum);
	whichbase = which_base (respnum);

	assert (whichbase < top_base);

	if (whichresp)
		sprintf(buf, txt_note_resp_page,
			whichbase + 1,
			top_base,
			whichresp,
			note_page + 1,
			note_h_subj);
	else
		sprintf(buf, txt_note_page,
			whichbase + 1,
			top_base,
			note_page + 1,
			note_h_subj);

	buf[COLS] = '\0';
	printf("%s\r\n\r\n", buf);

	note_line += 2;
}


void open_note (art, group_path)
	long art;
	char *group_path;
{
	char buf[1025];
	char *p;

	note_page = 0;

	if ((note_fp = open_art_fp (group_path, art)) == NULL) {
		note_page = NOTE_UNAVAIL;
		return;
	}

	note_h_from[0] = '\0';
	note_h_path[0] = '\0';
	note_h_subj[0] = '\0';
	note_h_org[0] = '\0';
	note_h_date[0] = '\0';
	note_h_newsgroups[0] = '\0';
	note_h_messageid[0] = '\0';
	note_h_distrib[0] = '\0';
	note_h_followup[0] = '\0';

	while (fgets(buf, 1024, note_fp) != NULL) {
		buf[1024] = '\0';

		for (p=buf ; *p && *p != '\n' ; p++) {
			if (((*p) & 0x7F) < 32)
				*p = ' ';
		}
		*p = '\0';
		
		if (*buf == '\0')
			break;

		if (strncmp(buf, "From: ", 6) == 0) {
			strcpy(note_h_from, &buf[6]);
			note_h_from[LEN-1] = '\0';
		} else if (strncmp(buf, "Path: ", 6) == 0) {
			strcpy(note_h_path, &buf[6]);
			note_h_path[LEN-1] = '\0';
		} else if (strncmp(buf, "Subject: ", 9) == 0) {
			strcpy(note_h_subj, &buf[9]);
			note_h_subj[LEN-1] = '\0';
		} else if (strncmp(buf, "Organization: ", 14) == 0) {
			strcpy(note_h_org, &buf[14]);
			note_h_org[LEN-1] = '\0';
		} else if (strncmp(buf, "Date: ", 6) == 0) {
			strcpy(note_h_date, &buf[6]);
			note_h_date[LEN-1] = '\0';
		} else if (strncmp(buf, "Newsgroups: ", 12) == 0) {
			strcpy(note_h_newsgroups, &buf[12]);
			note_h_newsgroups[LEN-1] = '\0';
		} else if (strncmp(buf, "Message-ID: ", 12) == 0) {
			strcpy(note_h_messageid, &buf[12]);
			note_h_messageid[LEN-1] = '\0';
		} else if (strncmp(buf, "Distribution: ", 14) == 0) {
			strcpy(note_h_distrib, &buf[14]);
			note_h_distrib[LEN-1] = '\0';
		} else if (strncmp(buf, "Followup-To: ", 13) == 0) {
			strcpy(note_h_followup, &buf[13]);
			note_h_followup[LEN-1] = '\0';
		}
	}

	note_page = 0;
	note_mark[0] = ftell (note_fp);

	parse_from (note_h_from, note_from_addr, note_full_name);
	note_end = FALSE;

	return;
}


void note_cleanup()
{
	if (note_page != NOTE_UNAVAIL)
		fclose(note_fp);
}


int prompt_response(ch, respnum)
	int respnum;
{
	int num;

	clear_message ();

	if ((num = parse_num (ch, txt_read_resp)) == -1) {
		clear_message ();
		return -1;
	}

	return choose_resp (which_base (respnum), num);
}

/*
 *  return response number n from thread i
 */

int choose_resp (i, n)
	int i;
	int n;
{
	int j;

	j = base[i];

	while (n-- && arts[j].thread >= 0) {
		j = arts[j].thread;
	}

	return j;
}


/*
 *  Parse various From: lines into the component mail addresses and
 *  real names
 */

void parse_from (str, addr, name)
	char *str;
	char *addr;
	char *name;
{
	char *p;
	
	for (p=str ; *p ; p++) {
		if (((*p) & 0x7F) < 32)
			*p = ' ';
	}

	while (*str && *str != ' ')
		*addr++ = *str++;
	*addr = '\0';
	if (*str++ == ' ') {
		if (*str++ == '(') {
			if (*str == '"')
				str++;  /* Kill "quotes around names"         */
					/* But don't touch quotes inside the  */
					/* Name (that's what that nonsense    */
					/* below is for			      */
			while (*str && *str != ')' && !(*str=='"'&&str[1]==')'))
				*name++ = *str++;
		}
	}
	*name = '\0';
}


/*
 *  Find the previous response.  Go to the last response in the previous
 *  thread if we go past the beginning of this thread.
 */

int prev_response (n)
	int n;
{
	int resp;
	int i;

	resp = which_resp (n);

	if (resp > 0)
		return choose_resp (which_base (n), resp-1);

	i = which_base (n) - 1;

	if (i < 0)
		return -1;

	return choose_resp (i, nresp (i));
}


/*
 *  Find the next response.  Go to the next basenote if there
 *  are no more responses in this thread
 */

int next_response (n)
	int n;
{
	int i;

	if (arts[n].thread >= 0)
		return arts[n].thread;

	i = which_base (n) + 1;

	if (i >= top_base)
		return -1;

	return base[i];
}


/*
 *  Given a respnum (index into arts[]), find the respnum of the
 *  next basenote
 */

int next_basenote (n)
	int n;
{
	int i;

	i = which_base (n) + 1;
	if (i >= top_base)
		return -1;

	return base[i];
}


void yank_to_addr (orig, addr)
	char *orig;
	char *addr;
{
	char *p;

	for (p = orig; *p; p++)
		if (((*p) & 0x7F) < 32)
			*p = ' ';

	while (*addr)
		addr++;

	while (*orig) {
		while (*orig && (*orig == ' ' || *orig == '"' || *orig == ','))
			orig++;
		*addr++ = ' ';
		while (*orig && (*orig != ' ' && *orig != ',' && *orig != '"'))
			*addr++ = *orig++;
		while (*orig && (*orig == ' ' || *orig == '"' || *orig == ','))
			orig++;
		if (*orig == '(') {
			while (*orig && *orig != ')')
				orig++;
			if (*orig == ')')
				orig++;
		}
	}
	*addr = '\0';
}

/*
 *  Read a file grabbing the address given for To: and
 *  sticking it in mail_to
 */

void find_new_to (nam, mail_to)
	char *nam;
	char *mail_to;
{
	FILE *fp;
	char buf[LEN+1];
	char buf2[LEN+1];
	char new_mail_to[LEN+1];
	char *p;

	*new_mail_to = '\0';

	if ((fp = fopen(nam, "r")) == NULL) {
		fprintf(stderr, txt_cannot_open, nam);
		return;
	}

	while (fgets(buf, 1024, fp) != NULL) {
		for (p = buf; *p && *p != '\n'; p++) ;
		*p = '\0';

		if (*buf == '\0')
			break;

		if (strncmp(buf, "To: ", 4) == 0) {
			strncpy(buf2, &buf[4], LEN);
			buf2[LEN-1] = '\0';
			yank_to_addr(buf2, new_mail_to);
		} else if (strncmp(buf, "Cc: ", 4) == 0) {
			strncpy(buf2, &buf[4], LEN);
			buf2[LEN-1] = '\0';
			yank_to_addr(buf2, new_mail_to);
		}
	}

	fclose(fp);
	if (new_mail_to[0] == ' ')
		my_strncpy(mail_to, &new_mail_to[1], LEN);
	else
		my_strncpy(mail_to, new_mail_to, LEN);
}


int mail_to_someone ()
{
	char nam[100];
	FILE *fp;
	char ch;
	char ch_default = 's';
	char buf[LEN+1];
	char mail_to[LEN+1];

	if (! parse_string (txt_mail_art_to, mail_to))
		return FALSE;
	if (mail_to[0] == '\0')
		return FALSE;

	setuid (real_uid);
	setgid (real_gid);

	sprintf(nam, "%s/.letter", homedir);
	if ((fp = fopen(nam, "w")) == NULL) {
		error_message (txt_cannot_open, nam);
		setuid(tin_uid);
		setgid(tin_gid);
		return FALSE;
	}
	chmod(nam, 0600);

	fprintf(fp, "To: %s\n", mail_to);
	fprintf(fp, "Subject: %s\n", note_h_subj);
	if (*note_h_followup)
		fprintf(fp, "Newsgroups: %s\n\n", note_h_followup);
	else
		fprintf(fp, "Newsgroups: %s\n", note_h_newsgroups);
	if (*my_org)
		fprintf(fp, "Organization: %s\n", my_org);
	if (*reply_to)
		fprintf(fp, "Reply-To: %s\n", reply_to);
	fputs("\n", fp);
	
	fseek(note_fp, 0L, 0);
	copy_fp(note_fp, fp, "");

	add_signature (fp, TRUE);
	fclose(fp);
	
	while (1) {
		do {
			sprintf (msg, "%s%c", txt_abort_edit_send, ch_default);
			wait_message (msg);
			MoveCursor(LINES, strlen (txt_abort_edit_send));
			if ((ch = ReadCh()) == CR)
				ch = ch_default;
		} while (ch != 'a' && ch != 'e' && ch != 's');

		switch (ch) {
		case 'e':
			start_line_offset = 5;
			invoke_editor(nam);
			break;

		case 'a':
			clear_message();
			setuid(tin_uid);
			setgid(tin_gid);
			return FALSE;

		case 's':
/*
 *  Open letter and get the To: line in case they changed it with
 *  the editor
 */
			find_new_to(nam, mail_to);
			sprintf (msg, txt_mailing_to, mail_to);
			wait_message (msg);
			sprintf(buf, "%s \"%s\" < %s", mailer, mail_to, nam);
			if (invoke_cmd(buf)) {
				info_message(txt_message_sent);
				goto mail_to_someone_done;
			} else {
				error_message (txt_command_failed_s, buf);
				break;
			}
		}
	}

mail_to_someone_done:
	setuid(tin_uid);
	setgid(tin_gid);

	return TRUE;
}


int mail_bug_report ()
{
	char nam[100];
	FILE *fp;
	char ch;
	char ch_default = 's';
	char buf[LEN+1];
	char mail_to[LEN+1];
	int start_line = 4;
	
	setuid (real_uid);
	setgid (real_gid);

	sprintf (nam, "%s/.bugreport", homedir);
	if ((fp = fopen (nam, "w")) == NULL) {
		error_message (txt_cannot_open, nam);
		setuid (tin_uid);
		setgid (tin_gid);
		return FALSE;
	}
	chmod(nam, 0600);

	fprintf (fp, "To: %s\n", BUG_REPORT_ADDRESS);
	fprintf (fp, "Subject: BUG REPORT %s %s\n", progname, version);
	if (*my_org) {
		fprintf (fp, "Organization: %s\n", my_org);
		start_line++;
	}
	if (*reply_to) {
		fprintf (fp, "Reply-To: %s\n", reply_to);
		start_line++;
	}
	fputs ("\n", fp);

	add_signature (fp, TRUE);
	fclose (fp);
	
	ch = 'e';
	while (1) {
		switch (ch) {
		case 'e':
			start_line_offset = start_line;
			invoke_editor (nam);
			break;

		case 'a':
			clear_message ();
			unlink (nam);
			setuid (tin_uid);
			setgid (tin_gid);
			return FALSE;

		case 's':
			strcpy (mail_to, BUG_REPORT_ADDRESS);
			find_new_to (nam, mail_to);
			sprintf (msg, txt_mailing_to, mail_to);
			wait_message (msg);
			sprintf (buf, "%s \"%s\" < %s", mailer, mail_to, nam);
			if (invoke_cmd (buf)) {
				info_message (txt_message_sent);
				goto mail_bug_report_done;
			} else {
				error_message (txt_command_failed_s, buf);
				break;
			}
		}

		do {
			sprintf (msg, "%s%c", txt_abort_edit_send, ch_default);
			wait_message (msg);
			MoveCursor (LINES, strlen (txt_abort_edit_send));
			if ((ch = ReadCh ()) == CR)
				ch = ch_default;
		} while (ch != 'a' && ch != 'e' && ch != 's');
	}

mail_bug_report_done:
	unlink (nam);
	
	setuid (tin_uid);
	setgid (tin_gid);

	return TRUE;
}


int mail_to_author (copy_text)
	int copy_text;
{
	char nam[100];
	FILE *fp;
	char ch;
	char ch_default = 's';
	char buf[LEN+1];
	char mail_to[LEN+1];

	setuid(real_uid);
	setgid(real_gid);

	sprintf(nam, "%s/.letter", homedir);
	if ((fp = fopen(nam, "w")) == NULL) {
		error_message (txt_cannot_open, nam);
		setuid(tin_uid);
		setgid(tin_gid);
		return(FALSE);
	}
	chmod(nam, 0600);

	fprintf(fp, "To: %s%s (%s)\n", note_from_addr, add_addr, note_full_name);
	fprintf(fp, "Subject: Re: %s\n", eat_re(note_h_subj) );
	fprintf(fp, "Newsgroups: %s\n", note_h_newsgroups);
	if (*my_org)
		fprintf(fp, "Organization: %s\n", my_org);
	if (*reply_to)
		fprintf(fp, "Reply-To: %s\n", reply_to);
	fputs("\n", fp);

	if (copy_text) {		/* if "copy_text" */
		fprintf(fp, txt_in_art_you_write, note_h_messageid);

		fseek(note_fp, note_mark[0], 0);
		copy_fp(note_fp, fp, "> ");
	}

	add_signature (fp, TRUE);
	fclose(fp);

	ch = 'e';
	while (1) {
		switch (ch) {
		case 'e':
			start_line_offset = 5;
			invoke_editor(nam);
			break;

		case 'a':
			clear_message();
			setuid(tin_uid);
			setgid(tin_gid);
			return FALSE;

		case 's':
			strcpy(mail_to, note_from_addr);
			find_new_to(nam, mail_to);
			sprintf(msg, txt_mailing_to, mail_to);
			wait_message (msg);
			sprintf (buf, "%s \"%s\" < %s", mailer, mail_to, nam);
			if (invoke_cmd (buf)) {
				info_message (txt_message_sent);
				goto mail_to_author_done;
			} else {
				error_message (txt_command_failed_s, buf);
				break;
			}
		}

		do {
			sprintf (msg, "%s%c", txt_abort_edit_send, ch_default);
			wait_message (msg);
			MoveCursor(LINES, strlen (txt_abort_edit_send));
			if ((ch = ReadCh()) == CR)
				ch = ch_default;
		} while (ch != 'a' && ch != 'e' && ch != 's');
	}

mail_to_author_done:
	setuid(tin_uid);
	setgid(tin_gid);

	return TRUE;
}


int post_response (group, respnum)
	char *group;
	int respnum;
{
	FILE *fp;
	char nam[LEN+1];
	char ch, *ptr;
	char ch_default = 'p';
	char buf[LEN+1];

	start_line_offset = 6;

	if (*note_h_followup && strcmp (note_h_followup, "poster") == 0) {
		clear_message ();
		if (! prompt_yn (LINES, txt_resp_to_poster, 'y')) {
			return FALSE;
		}
		*note_h_followup = '\0';
	} else if (*note_h_followup && strcmp(note_h_followup, group) != 0) {
	    MoveCursor (LINES/2, 0);
	    CleartoEOS ();
		StartInverse ();
		center_line ((LINES/2)+2, txt_resp_redirect);
		EndInverse ();
	    MoveCursor ((LINES/2)+4, 0);

		printf ("    ");
		ptr = note_h_followup;
		while (*ptr) {
			if (*ptr != ',') {
				putc (*ptr, stdout);
			} else {
				printf ("\r\n    ");
			}
			fflush (stdout);
			ptr++;
		}

		if (! prompt_yn (LINES, txt_continue, 'y')) {
			return FALSE;
		}
	}

	setuid(real_uid);
	setgid(real_gid);

	sprintf(nam, "%s/.article", homedir);
	if ((fp = fopen(nam, "w")) == NULL) {
		error_message (txt_cannot_open, nam);
		setuid(tin_uid);
		setgid(tin_gid);
		return FALSE;
	}
	chmod(nam, 0600);

	fprintf(fp, "Subject: Re: %s\n", eat_re(note_h_subj));

	if (*note_h_followup && strcmp(note_h_followup, "poster") != 0)
		fprintf(fp, "Newsgroups: %s\n", note_h_followup);
	else
		fprintf(fp, "Newsgroups: %s\n", note_h_newsgroups);

	if (*my_org) {
		fprintf(fp, "Organization: %s\n", my_org);
	}

	if (*reply_to) {
		fprintf(fp, "Reply-To: %s\n", reply_to);
		start_line_offset++;
	}

	if (note_h_distrib != '\0') {
		fprintf(fp, "Distribution: %s\n", note_h_distrib);
		start_line_offset++;
	}

	fprintf(fp, "References: %s\n", note_h_messageid);
	fprintf(fp, "\n");

	if (respnum) {		/* if "copy_text" */
		if (note_h_from[0]) { 	
			fprintf(fp, txt_writes, note_h_from);
		}

		fseek(note_fp, note_mark[0], 0);
		copy_fp(note_fp, fp, "> ");
	}

	add_signature (fp, FALSE);
	fclose(fp);

	ch = 'e';
	while (1) {
		switch (ch) {
		case 'e':
			invoke_editor (nam);
			break;

		case 'a':
			clear_message ();
			setuid (tin_uid);
			setgid (tin_gid);
			return FALSE;

		case 'p':
			wait_message (txt_posting);
			sprintf (buf, "%s/inews -h < %s", LIBDIR, nam);
			if (invoke_cmd (buf)) {
				info_message (txt_art_posted);
				goto post_response_done;
			} else {
				error_message (txt_art_rejected, NULL);
				break;
			}
		}

		do {
			sprintf (msg, "%s%c", txt_abort_edit_post, ch_default);
			wait_message (msg);
			MoveCursor(LINES, strlen (txt_abort_edit_post));
			if ((ch = ReadCh()) == CR)
				ch = ch_default;
		} while (ch != 'a' && ch != 'e' && ch != 'p');
	}

post_response_done:
	setuid(tin_uid);
	setgid(tin_gid);

	if (*note_h_followup && strcmp(note_h_followup, "poster") != 0)
		update_art_posted_file (note_h_followup, note_h_subj);
	else
		update_art_posted_file (note_h_newsgroups, note_h_subj);

	return TRUE;
}


void pipe_article()
{
	char command[LEN+1];
	FILE *fp;

	if (!parse_string(txt_pipe_to_command, command))
		return;
	if (command[0] == '\0')
		return;

	Raw(FALSE);

	if ((fp = popen(command, "w")) == NULL) {
		error_message (txt_command_failed_s, command);
		goto pipe_article_done;
	}

	fseek(note_fp, 0L, 0);
	copy_fp(note_fp, fp, "");
	pclose(fp);

pipe_article_done:

	Raw(TRUE);
	continue_prompt();
}


void print_article ()
{
	char command[LEN+1];
	FILE *fp;
	
	if (default_printer) {
		sprintf (command, "%s -P%s > /dev/null 2>&1",
			printer, get_val ("PRINTER","ps0"));
	} else {
		sprintf (command, "%s > /dev/null 2>&1", printer);
	}
	
	if ((fp = popen(command, "w")) == NULL) {
		info_message (txt_error_printing_art);
	} else {
		wait_message (txt_printing);
		if (print_header) {
			fseek(note_fp, 0L, 0);
		} else {
			fprintf (fp, "From: %s\n", note_h_from);
			fprintf (fp, "Subject: %s\n\n", note_h_subj);
			fseek (note_fp, note_mark[0], 0);
		}
		copy_fp(note_fp, fp, "");
		pclose(fp);
		fseek(note_fp, note_mark[note_page], 0);
		info_message (txt_art_printed);
	}
}


void print_thread (respnum, group_path)
	int respnum;
	char *group_path;
{
	char command[LEN+1];
	FILE *fp;
	int b;
	int i;
	
	b = which_base (respnum);

	if (default_printer) {
		sprintf (command, "%s -P%s > /dev/null 2>&1",
			printer, get_val ("PRINTER","ps0"));
	} else {
		sprintf (command, "%s > /dev/null 2>&1", printer);
	}
	
	for (i = base[b]; i >= 0; i = arts[i].thread) {
		if ((fp = popen(command, "w")) == NULL) {
			info_message (txt_error_printing_thread);
			break;
		} else {
			note_cleanup();
			wait_message (txt_printing);
			open_note(arts[i].artnum, group_path);	

			if (print_header) {
				fseek(note_fp, 0L, 0);
			} else {
				fprintf (fp, "From: %s\n", note_h_from);
				fprintf (fp, "Subject: %s\n\n", note_h_subj);
				fseek (note_fp, note_mark[0], 0);
			}
			copy_fp(note_fp, fp, "");

			pclose(fp);
		}
		info_message (txt_thread_printed);
	}

	open_note(arts[respnum].artnum, group_path);	
	fseek(note_fp, note_mark[note_page], 0);
}


int show_last_page ()
{
	char buf[LEN+1];
	char buf2[LEN+50];
	char *p, *q;
	int ctrl_L;		/* form feed character detected */
	int i, j;
	
	if (note_end) {
		return FALSE;
	}

	while (! note_end) {
		note_line = 1;
		ctrl_L = FALSE;

		if (note_page == 0) {
			note_line += 4;
		} else {
			note_line += 2;
		}
		while (note_line < LINES) {
			if (fgets(buf, LEN, note_fp) == NULL) {
				note_end = TRUE;
				break;
			}
			buf[LEN-1] = '\0';
			for (p = buf, q = buf2;	*p && *p != '\n' && q<&buf2[LEN]; p++) {
				if (*p == '\b' && q > buf2) {
					q--;
				} else if (*p == 12) {		/* ^L */
					*q++ = '^';
					*q++ = 'L';
					ctrl_L = TRUE;
				} else if (*p == '\t') {
					i = q - buf2;
					j = (i|7) + 1;

					while (i++ < j) {
						*q++ = ' ';
					}
				} else if (((*p) & 0x7F) < 32) {
					*q++ = '^';
					*q++ = ((*p) & 0x7F) + '@';
				} else {
					*q++ = *p;
				}
			}
			*q = '\0';
			note_line += ((int) strlen (buf2) / COLS) + 1;

			if (ctrl_L) {
				break;
			}
		}
		if (! note_end) {
			note_mark[++note_page] = ftell(note_fp);
		}
	}
	fseek (note_fp, note_mark[note_page], 0);
	return TRUE;
}
