/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : prompt.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 10-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

/*
 *  parse_num
 *  get a number from the user
 *  Return -1 if missing or bad number typed
 */

int parse_num (ch, prompt)
	char ch;
	char *prompt;
{
	char buf[40];
	int len;
	int i;
	int num;

	MoveCursor(LINES,0);
	printf("%s %c",prompt,ch);
	fflush(stdout);
	buf[0] = ch;
	buf[1] = '\0';
	len = 1;
	ch = ReadCh();
	while (ch != '\n' && ch != '\r') {
		if (ch == 8 || ch == 127) {
			if (len) {
				len--;
				buf[len] = '\0';
				putchar('\b');
				putchar(' ');
				putchar('\b');
			} else {
				MoveCursor(LINES, 0);
				CleartoEOLN();
				return(-1);
			}
		} else if (ch == 21) {	/* control-U	*/
			for (i = len;i>0;i--) {
				putchar('\b');
				putchar(' ');
				putchar('\b');
			}
			buf[0] = '\0';
			len = 0;
		} else if (ch >= '0' && ch <= '9' && len < 4) {
			buf[len++] = ch;
			buf[len] = '\0';
			putchar (ch);
		} else
			putchar(7);
		fflush(stdout);
		ch = ReadCh();
	}

	MoveCursor(LINES, 0);
	CleartoEOLN();

	if (len) {
		num = atoi(buf);
		return(num);
	} else
		return(-1);
}


/*
 *  parse_string
 *  get a string from the user
 *  Return TRUE if a valid string was typed, FALSE otherwise
 */

int parse_string (prompt, buf)
	char *prompt;
	char *buf;
{
	int len;
	int i;
	char ch;

	clear_message();
	MoveCursor(LINES,0);
	printf("%s", prompt);
	fflush(stdout);
	buf[0] = '\0';
	len = 0;
	ch = ReadCh();
	while (ch != '\n' && ch != '\r') {
		if (ch == 8 || ch == 127) {
			if (len) {
				len--;
				buf[len] = '\0';
				putchar('\b');
				putchar(' ');
				putchar('\b');
			} else {
				MoveCursor(LINES, 0);
				CleartoEOLN();
				return(FALSE);
			}
		} else if (ch == 21) {	/* control-U	*/
			for (i = len;i>0;i--) {
				putchar('\b');
				putchar(' ');
				putchar('\b');
			}
			buf[0] = '\0';
			len = 0;
		} else if (ch >= ' ' && len < 60) {
			buf[len++] = ch;
			buf[len] = '\0';
			putchar (ch);
		} else
			putchar(7);
		fflush(stdout);
		ch = ReadCh();
	}
	MoveCursor(LINES,0);
	CleartoEOLN();

	return TRUE;
}


int prompt_yn (line, prompt, default_ch)
	int line;
	char *prompt;
	char default_ch;
{
	char ch;

	MoveCursor (line, 0);
	CleartoEOLN ();
	printf ("%s%c", prompt, default_ch);
	fflush (stdout);
	MoveCursor (line, strlen (prompt));

	if ((ch = ReadCh()) == CR) {
		ch = default_ch;
	}	

	if (line == LINES) {
		clear_message();
	} else {
		MoveCursor (line, strlen (prompt));
		printf ("%c", ch);
		fflush (stdout);
	}

	return (ch == 'y' ? TRUE : FALSE);
}


void continue_prompt()
{
	info_message(txt_hit_any_key);
	ReadCh();
}


