/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : rcfile.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 18-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

#define COL2	COLS/2

extern char index_file[LEN+1];

/*
 *  read_rcfile - read defaults from ~/.tin/tinrc
 */

int read_rcfile ()
{
	char buf[LEN+1];
	FILE *fp;

	if ((fp = fopen (rcfile, "r")) != NULL) {
		while (fgets (buf, LEN, fp) != NULL) {
			if (buf[0] != '#') { 
				if (strncmp (buf, "save_archive=", 13) == 0) {
					save_archive_name = (strncmp (&buf[13], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "save_separate=", 14) == 0) {
					save_separate = (strncmp (&buf[14], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "mark_saved_read=", 16) == 0) {
					mark_saved_read = (strncmp (&buf[16], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "kill_articles=", 14) == 0) {
					kill_articles = (strncmp (&buf[14], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "show_author=", 12) == 0) {
					show_author = (strncmp (&buf[12], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "draw_arrow=", 11) == 0) {
					draw_arrow_mark = (strncmp (&buf[11], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "post_process=", 13) == 0) {
					post_process = (strncmp (&buf[13], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "print_header=", 13) == 0) {
					print_header = (strncmp (&buf[14], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "post_process_type=", 18) == 0) {
					post_proc_type = atoi (&buf[18]);
				} else if (strncmp (buf, "sort_article_type=", 18) == 0) {
					sort_art_type = atoi (&buf[18]);
				} else if (strncmp (buf, "savedir=", 8) == 0) {
					strncpy (savedir, &buf[8], LEN);
					savedir[strlen (savedir) - 1] = '\0';
					if (savedir[0] == '.' && strlen (savedir) == 1) {
#ifdef BSD
						getwd (buf);	
#else
						getcwd (buf, LEN);
#endif
						my_strncpy (savedir, buf, LEN);
					} /*  else if (savedir[0] == '.' && savedir[1] == '.' && strlen (savedir) == 2) {
					} */
				} else if (strncmp (buf, "maildir=", 8) == 0) {
					strncpy (maildir, &buf[8], LEN);
					maildir[strlen (maildir) - 1] = '\0';
				} else if (strncmp (buf, "printer=", 8) == 0) {
					strncpy (printer, &buf[8], LEN);
					printer[strlen (printer) - 1] = '\0';
				} else if (strncmp (buf, "spooldir=", 9) == 0) {
					strncpy (spooldir, &buf[9], LEN);
					spooldir[strlen (spooldir) - 1] = '\0';
				} else if (strncmp (buf, "signature=", 10) == 0) {
					strncpy (signature, &buf[10], LEN);
					signature[strlen (signature) - 1] = '\0';
				} else if (strncmp (buf, "sig=", 4) == 0) {
					strncpy (sig, &buf[4], LEN);
					sig[strlen (sig) - 1] = '\0';
				}
			}
		}
		fclose (fp);
		return TRUE;		
	}
	return FALSE;		
}

/*
 *  write_rcfile - write defaults to ~/.tin/tinrc
 */

void write_rcfile ()
{
	FILE *fp;

	if ((fp = fopen (rcfile, "w")) != NULL) {
		wait_message (txt_saving);

		fprintf (fp, "# (-a) if ON articles/threads with Archive-name: in mail header will\n");
		fprintf (fp, "# be automatically saved with the Archive-name & part/patch no.\n");
		fprintf (fp, "save_archive=%s\n\n", (save_archive_name ? "ON" : "OFF"));
		fprintf (fp, "# (-s) if ON articles of a threads will be saved to separate files\n");
		fprintf (fp, "# otherwise the whole thread will be saved to one file\n");
		fprintf (fp, "save_separate=%s\n\n", (save_separate ? "ON" : "OFF"));
		fprintf (fp, "# if ON mark articles that are saved as read\n");
		fprintf (fp, "mark_saved_read=%s\n\n", (mark_saved_read ? "ON" : "OFF"));
		fprintf (fp, "# if ON show Subject & From otherwise just Subject\n");
		fprintf (fp, "show_author=%s\n\n", (show_author ? "ON" : "OFF"));
		fprintf (fp, "# if ON use -> otherwise highlighted bar for selection\n");
		fprintf (fp, "draw_arrow=%s\n\n", (draw_arrow_mark ? "ON" : "OFF"));
		fprintf (fp, "# if ON kill articles that match kill file\n");
		fprintf (fp, "kill_articles=%s\n\n", (kill_articles ? "ON" : "OFF"));
		fprintf (fp, "# if ON save articles/threads to subdir with Archive-name: under -d directory\n");
		fprintf (fp, "print_header=%s\n\n", (print_header ? "ON" : "OFF"));
		fprintf (fp, "# if ON post process saved articles/threads\n");
		fprintf (fp, "post_process=%s\n\n", (post_process ? "ON" : "OFF"));
		fprintf (fp, "# type of post processing to perform\n");
		fprintf (fp, "post_process_type=%d\n\n", post_proc_type);
		fprintf (fp, "# sort articles by 0) nothing 1) subject 2) from or 3) date field\n");
		fprintf (fp, "sort_article_type=%d\n\n", sort_art_type);
		fprintf (fp, "# (-d) directory where articles/threads are saved\n");
		fprintf (fp, "savedir=%s\n\n", savedir);
		fprintf (fp, "# (-M) directory where articles/threads are saved in mailbox format\n");	
		fprintf (fp, "maildir=%s\n\n", maildir);	
		fprintf (fp, "# (-p) print program with parameters used to print articles/threads\n");
		fprintf (fp, "printer=%s\n\n", printer);
		fprintf (fp, "# (-S) directory where news is spooled\n");
		fprintf (fp, "spooldir=%s\n\n", spooldir);

		fprintf (fp, "# .signature file used for replies, followups\n");
		fprintf (fp, "signature=%s\n\n", signature);
		fprintf (fp, "# .Sig file used for postings\n");
		fprintf (fp, "sig=%s\n\n", sig);

		fclose (fp);
		chmod (rcfile, 0600);
	}
}

/*
 *  options menu so that the user can dynamically change parameters
 */
 
int change_rcfile (group, kill_at_once)
	char *group;
	int kill_at_once;
{
	char *str;
	int ch;
	int kill_changed = FALSE;
	int orig_kill_state;
	int option;
	int ret_code = NO_KILLING;
	
	ClearScreen ();
	printf("%s\r\n", nice_time());	/* time in upper left */

	StartInverse ();
	center_line (1, txt_options_menu);
	EndInverse ();
	
	MoveCursor(INDEX_TOP, 0);
	printf ("%s%s\r\n\r\n", txt_opt_autosave, (save_archive_name ? "ON " : "OFF"));
	printf ("%s%s\r\n\r\n", txt_opt_mark_saved_read, (mark_saved_read ? "ON " : "OFF"));
	printf ("%s%s\r\n\r\n", txt_opt_show_author, (show_author ? "ON " : "OFF"));
	printf ("%s%s", txt_opt_post_process, (post_process ? "ON " : "OFF"));

	MoveCursor(INDEX_TOP, COL2);
	printf ("%s%s", txt_opt_save_separate, (save_separate ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+2, COL2);
	printf ("%s%s", txt_opt_kill_articles, (kill_articles ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+4, COL2);
	printf ("%s%s", txt_opt_draw_arrow, (draw_arrow_mark ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+6, COL2);
	printf ("%s%s", txt_opt_print_header, (print_header ? "ON " : "OFF"));


	MoveCursor(INDEX_TOP+8, 0);
	switch (post_proc_type) {
		case POST_PROC_SH:
			str = txt_post_process_sh;
			break;
		case POST_PROC_UUD:
			str = txt_post_process_uud;
			break;
		case POST_PROC_UUD_ZOO:
			str = txt_post_process_uud_zoo;
			break;
		case POST_PROC_UUD_LZH:
			str = txt_post_process_uud_lzh;
			break;
		case POST_PROC_UUD_ARC:
			str = txt_post_process_uud_arc;
			break;
		case POST_PROC_UUD_ZIP:
			str = txt_post_process_uud_zip;
			break;
		case POST_PROC_PATCH:
			str = txt_post_process_patch;
			break;
	}
	printf ("%s%s\r\n\r\n", txt_opt_process_type, str);
	
	MoveCursor(INDEX_TOP+10, 0);
	switch (sort_art_type) {
		case SORT_BY_NONE:
			str = txt_sort_by_none;
			break;
		case SORT_BY_SUBJ:
			str = txt_sort_by_subj;
			break;
		case SORT_BY_FROM:
			str = txt_sort_by_from;
			break;
		case SORT_BY_DATE:
			str = txt_sort_by_date;
			break;
	}
	printf ("%s%s\r\n\r\n", txt_opt_sort_type, str);

	printf ("%s%s\r\n\r\n", txt_opt_savedir, savedir);
	printf ("%s%s\r\n\r\n", txt_opt_maildir, maildir);
	printf ("%s%s\r\n\r\n", txt_opt_printer, printer);
	fflush(stdout);

	while (1) {
		show_menu_help ("Select option by entering number before text. Enter '0' to save/quit.");
		MoveCursor (LINES, 0);
		ch = ReadCh ();
		if (ch == '0') {
			option = 0;
		} else {
			option = parse_num (ch, "Enter option number> ");
		}
	switch (option) {
		case 0:
			write_rcfile ();
			if (kill_changed) {
				if (kill_at_once) {
					if (kill_articles) {
						read_kill_file ();
						if (kill_any_articles (group)) {
							reload_index_file (group, TRUE);	/* kill arts */
						}
					} else {
						reload_index_file (group, FALSE);	/* add killed arts */
					}
				}
				ret_code = KILLING;
			}
			clear_note_area ();
			return ret_code;
			
		case 1:		/* auto save */
			show_menu_help (txt_help_autosave);
			do {
				MoveCursor (INDEX_TOP, strlen (txt_opt_autosave));
				if ((ch = ReadCh()) == ' ') {
					save_archive_name = !save_archive_name;
					printf ("%s", (save_archive_name ? "ON " : "OFF"));
					fflush(stdout);
				}
			} while (ch != CR);
			break;

		case 2:		/* save sperate */
			show_menu_help (txt_help_save_separate);
			do {
				MoveCursor (INDEX_TOP, COL2 + (int) strlen (txt_opt_save_separate));
				if ((ch = ReadCh()) == ' ') {
					save_separate = !save_separate;
					printf ("%s", (save_separate ? "ON " : "OFF"));
					fflush(stdout);
				}
			} while (ch != CR);
			break;
			
		case 3:		/* mark saved articles read */
			show_menu_help (txt_help_mark_saved_read);
			do {
				MoveCursor (INDEX_TOP+2, strlen (txt_opt_mark_saved_read));
				if ((ch = ReadCh()) == ' ') {
					mark_saved_read = !mark_saved_read;
					printf ("%s", (mark_saved_read ? "ON " : "OFF"));
					fflush(stdout);
				}
			} while (ch != CR);
			break;

		case 4:		/* kill articles */
			orig_kill_state = kill_articles;
			show_menu_help (txt_help_kill_articles);
			do {
				MoveCursor (INDEX_TOP+2, COL2 + (int) strlen (txt_opt_kill_articles));
				if ((ch = ReadCh()) == ' ') {
					kill_articles = !kill_articles;
					kill_changed = (kill_articles != orig_kill_state ? TRUE : FALSE);
					printf ("%s", (kill_articles ? "ON " : "OFF"));
					fflush(stdout);
				}
			} while (ch != CR);
			break;

		case 5:		/* show subject & author / subject only */
			show_menu_help (txt_help_show_author);
			do {
				MoveCursor (INDEX_TOP+4, strlen (txt_opt_show_author));
				if ((ch = ReadCh()) == ' ') {
					show_author = !show_author;
					printf ("%s", (show_author ? "ON " : "OFF"));
					fflush(stdout);	
				}
			} while (ch != CR);
			break;
			
		case 6:		/* draw -> / highlighted bar */
			show_menu_help (txt_help_draw_arrow);
			do {
				MoveCursor (INDEX_TOP+4, COL2 + (int) strlen (txt_opt_draw_arrow));
				if ((ch = ReadCh()) == ' ') {
					draw_arrow_mark = !draw_arrow_mark;
					printf ("%s", (draw_arrow_mark ? "ON " : "OFF"));
					fflush(stdout);
				}
			} while (ch != CR);
			break;

		case 7:		/* post process saved files */
			show_menu_help (txt_help_post_process);
			do {
				MoveCursor (INDEX_TOP+6, strlen (txt_opt_post_process));
				if ((ch = ReadCh()) == ' ') {
					post_process = !post_process;
					printf ("%s", (post_process ? "ON " : "OFF"));
					fflush(stdout);
				}
			} while (ch != CR);
			break;
			
		case 8:		/* print header */
			show_menu_help (txt_help_print_header);
			do {
				MoveCursor (INDEX_TOP+6, COL2 + (int) strlen (txt_opt_print_header));
				if ((ch = ReadCh()) == ' ') {
					print_header = !print_header;
					printf ("%s", (print_header ? "ON " : "OFF"));
					fflush(stdout);
				}
			} while (ch != CR);
			break;
			
		case 9:
			show_menu_help (txt_help_post_proc_type);
			do {
				MoveCursor (INDEX_TOP+8, strlen (txt_opt_process_type));
				if ((ch	= ReadCh()) == ' ') {
					if (post_proc_type + 1 > POST_PROC_PATCH) {
						post_proc_type = POST_PROC_SH;
					} else {
						post_proc_type++;
					}
					switch (post_proc_type) {
						case POST_PROC_SH:
							str = txt_post_process_sh;
							break;
						case POST_PROC_UUD:
							str = txt_post_process_uud;
							break;
						case POST_PROC_UUD_ZOO:
							str = txt_post_process_uud_zoo;
							break;
						case POST_PROC_UUD_LZH:
							str = txt_post_process_uud_lzh;
							break;
						case POST_PROC_UUD_ARC:
							str = txt_post_process_uud_arc;
							break;
						case POST_PROC_UUD_ZIP:
							str = txt_post_process_uud_zip;
							break;
						case POST_PROC_PATCH:
							str = txt_post_process_patch;
							break;
					}
					CleartoEOLN (); 
					printf ("%s", str);
					fflush(stdout);
				}
			} while (ch != CR);
			break;
		case 10:
			show_menu_help (txt_help_sort_type);
			do {
				MoveCursor (INDEX_TOP+10, strlen (txt_opt_sort_type));
				if ((ch	= ReadCh()) == ' ') {
					if (sort_art_type + 1 > SORT_BY_DATE) {
						sort_art_type = SORT_BY_NONE;
					} else {
						sort_art_type++;
					}
					switch (sort_art_type) {
						case SORT_BY_NONE:
							str = txt_sort_by_none;
							break;
						case SORT_BY_SUBJ:
							str = txt_sort_by_subj;
							break;
						case SORT_BY_FROM:
							str = txt_sort_by_from;
							break;
						case SORT_BY_DATE:
							str = txt_sort_by_date;
							break;
					}
					CleartoEOLN (); 
					printf ("%s", str);
					fflush(stdout);
				}
			} while (ch != CR);
			break;
		case 11:
			show_menu_help (txt_help_savedir);
			parse_menu_string (INDEX_TOP+12, strlen (txt_opt_savedir), savedir);
			expand_rel_abs_pathname (INDEX_TOP+12, strlen (txt_opt_savedir), savedir);
			break;
		case 12:
			show_menu_help (txt_help_maildir);
			parse_menu_string (INDEX_TOP+14, strlen (txt_opt_maildir), maildir);
			expand_rel_abs_pathname (INDEX_TOP+14, strlen (txt_opt_maildir), maildir);
			break;
		case 13:
			show_menu_help (txt_help_printer);
			parse_menu_string (INDEX_TOP+16, strlen (txt_opt_printer), printer);
			expand_rel_abs_pathname (INDEX_TOP+16, strlen (txt_opt_printer), printer);
			break;
	}
	}
}

/*
 *  parse_menu_string
 *  get a string from the user
 *  Return TRUE if a valid string was typed, FALSE otherwise
 */

void parse_menu_string (line, col, var)
	int line;
	int col;
	char *var;
{
	char buf[LEN+1];
	char ch;
	int len;
	int i;

	MoveCursor (line, col);
	buf[0] = '\0';
	len = 0;
	ch = ReadCh();
	while (ch != '\n' && ch != '\r') {
		if (ch == 8 || ch == 127) {
			if (len) {
				len--;
				buf[len] = '\0';
				putchar('\b');
				putchar(' ');
				putchar('\b');
			} else {
				strcpy (var, buf);
				MoveCursor(line, col);
				CleartoEOLN();
			}
		} else if (ch == 21) {	/* control-U	*/
			for (i = len;i>0;i--) {
				putchar('\b');
				putchar(' ');
				putchar('\b');
			}
			buf[0] = '\0';
			len = 0;
		} else if (ch >= ' ' && len < 60) {
			buf[len++] = ch;
			buf[len] = '\0';
			putchar (ch);
		} else
			putchar(7);
		fflush(stdout);
		ch = ReadCh();
	}

	if (buf[0]) {
		strcpy (var, buf);
	}
}

/*
 *  expand ~/News to /usr/username/News and print to screen
 */
 
void expand_rel_abs_pathname (line, col, str)
	int line;
	int col;
	char *str;
{
	char buf[LEN+1];
	
	if (str[0] == '~') {
		if (strlen (str) == 1) {
			strcpy (str, homedir);
		} else {
			sprintf (buf, "%s%s", homedir, str+1);
			strcpy (str, buf);
		}
	}
	MoveCursor (line, col);
	CleartoEOLN ();
	puts (str);
	fflush (stdout);
}

/*
 *  show_menu_help
 */
 
void show_menu_help (help_message)
	char *help_message;
{
	 MoveCursor (INDEX_TOP+18, 0);
	 CleartoEOLN ();
	 center_line (INDEX_TOP+18, help_message);
}
