/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : save.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 23-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta, Iain Lea & Wayne Brandt
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

#define	INITIAL		1
#define MIDDLE		2
#define OFF			3
#define END			4

int create_subdir = TRUE;

struct save_t *save;
int save_num=0;
int max_save;

/*
 * types of archive programs
 * 0=archiver, 1=extension, 2=extract option, 3=list option
 */
 
struct archiver_t { 
	char *name;
	char *ext;
	char *extract;
	char *list;
};

struct archiver_t archiver[] = {
	{ "",      "",    "",         "" },
	{ "",      "",    "",         "" },
	{ "",      "",    "",         "" },
	{ "zoo",   "zoo", "-extract", "-list" },
	{ "lharc", "lzh", "x",        "l" },
	{ "arc",   "arc", "x",        "l" },
	{ "zip",   "zip", "-x",       "-l" },
	{ "",      "",    "",         "" }
};

extern char *glob_page_group;
extern char note_h_path[LEN+1];	/* Path:	*/
extern char note_h_date[LEN+1];	/* Date:	*/
extern FILE	*note_fp;			/* the body of the current article */
extern int index_point;
extern int note_end;
extern int note_page;
extern long note_mark[MAX_PAGES];

char default_regex_pattern[LEN+1];


void save_art_to_file (respnum, index, mailbox, filename)
	int respnum;
	int index;
	int mailbox;
	char *filename;
{
	char file[LEN+1], *p;
	char save_art_info[LEN+1];
	FILE *fp;
	int is_mailbox = FALSE;
	int i = 0;
	
	file[0] = '\0';
	
	if (filename) {
		my_strncpy (file, filename, LEN);
		is_mailbox = mailbox;
		i = index;
	} else {
		if (save_archive_name && arts[respnum].archive) {
			my_strncpy (file, arts[respnum].archive, LEN);
		} else {
			if (! parse_string(txt_save_art_to_file, file)) {
				return;
			}
			if (file[0] == '\0') {
				info_message (txt_no_filename);
				return;
			}

			for (p = file; *p && (*p == ' ' || *p == '\t'); p++) {
				;
			}
			if (! *p) {
				return;
			}

			is_mailbox = create_path (file);
		}
		free_save_array ();
		add_to_save_list (save_num, &arts[respnum], is_mailbox, file);
	}

	if (! append_to_existing_file (i)) {
		info_message(txt_art_not_saved);
		sleep (1);
		return;
	}

	setuid(real_uid);
	setgid(real_gid);

	if ((fp = fopen(save_filename (i), "a+")) == NULL) {
		info_message(txt_art_not_saved);
		setuid(real_uid);
		setgid(real_gid);
		return;
	}

	if (! filename) {
		wait_message (txt_saving);
	}
	
	fprintf(fp, "From %s %s\n", note_h_path, note_h_date);

	if (fseek(note_fp, 0L, 0) == -1) {
		error_message ("fseek error on \"%s\"", arts[respnum].subject);
	}
	copy_fp(note_fp, fp, "");
	fputs("\n", fp);
	fclose(fp);
	fseek(note_fp, note_mark[note_page], 0);

	save[i].saved = TRUE;

	if (mark_saved_read) {
		arts[respnum].unread = ART_READ;
	}
	
	setuid(real_uid);
	setgid(real_gid);

	if (! filename) {
		if (is_mailbox) {
			sprintf (save_art_info, txt_saved_to_mailbox, get_first_savefile ());
		} else {
			sprintf (save_art_info, txt_art_saved_to, get_first_savefile ());
		}
		info_message(save_art_info);
	}
}


void save_thread_to_file (respnum, group_path)
	int respnum;
	char *group_path;
{
	char *p;
	char file[LEN+1];
	char save_thread_info[LEN+1];
	char *first_savefile;
	FILE *fp;
	int count = 0;
	int is_mailbox = FALSE;
	int b, i;
	int orig_note_end;
	int orig_note_page;

	file[0] = '\0';
	free_save_array ();

	b = which_base (respnum);
	
	if (! save_archive_name || ! arts[base[b]].archive) {
		if (! parse_string(txt_save_thread_to, file)) {
			return;
		}
		if (file[0] == '\0') {
			info_message (txt_no_filename);
			return;
		}

		for (p = file; *p && (*p == ' ' || *p == '\t'); p++) {
			;
		}
		if (! *p) {
			return;
		}

		is_mailbox = create_path (file);
	}
	
	setuid (real_uid);
	setgid (real_gid);

	for (i = base[b] ; i >= 0 ; i = arts[i].thread) {
		add_to_save_list (i, &arts[i], is_mailbox, file);
	}
	sort_save_list ();

	wait_message (txt_saving);
	
	orig_note_end = note_end;
	orig_note_page = note_page;
	note_cleanup ();		/* close currently opened article */

	for (i=0 ; i < save_num ; i++) {
		if (! append_to_existing_file (i)) {
			info_message(txt_art_not_saved);
			sleep (1);
			setuid(real_uid);
			setgid(real_gid);
			continue;
		} else {
			wait_message (txt_saving);
		}

		if ((fp = fopen (save_filename (i), "a+")) == NULL) {
			info_message(txt_thread_not_saved);
			goto reopen_article;
		}	

		open_note (arts[save[i].index].artnum, group_path);
		fprintf (fp, "From %s %s\n", note_h_path, note_h_date);
		fseek (note_fp, 0L, 0);
		copy_fp (note_fp, fp, "");
		note_cleanup ();
		save[i].saved = TRUE;

		if (mark_saved_read) {
			arts[save[i].index].unread = ART_READ;
		}

		printf("\b\b\b\b%4d", ++count);
		fflush(stdout);
		fclose(fp);
	}
	
	if (! (first_savefile = get_first_savefile ())) {
		info_message(txt_thread_not_saved);
	} else {
		if (is_mailbox) {
			sprintf (save_thread_info, txt_saved_to_mailbox, first_savefile);
		} else {
			if (save_num == 1) {
				sprintf (save_thread_info, txt_art_saved_to, first_savefile);
			} else {
				if (save_separate) {
					sprintf (save_thread_info, txt_thread_saved_to_many,
						first_savefile, get_last_savefile ());
				} else {
					sprintf (save_thread_info, txt_thread_saved_to,
						first_savefile);
				}
			}
			free (first_savefile);
			first_savefile = '\0';
		}
		info_message(save_thread_info);
	}
	
reopen_article:
	setuid(real_uid);
	setgid(real_gid);
	open_note(arts[respnum].artnum, group_path);
	note_end = orig_note_end;
	note_page = orig_note_page;
	fseek (note_fp, note_mark[note_page], 0);
}


int save_regex_arts (group_path, group)
	char *group_path;
	char *group;
{
	char buf[LEN+1];
	char buf2[LEN+1];
	char dir[LEN+1];
	char file[LEN+1];
	char mailbox[LEN+1];
	int i, j;	
	int	is_mailbox = FALSE;
	int save_tagged_files = FALSE;
	
	if (index_point < 0) {
		info_message (txt_no_match);
		return FALSE; 
	}
	
	free_save_array ();
	clear_message ();
	
	/*
	 *  check if tagged files and if not ask user for subject pattern
	 */
	if (num_of_tagged_files) {
		if (prompt_yn (LINES, "Enter 'y' to save tagged files or 'n' to enter regex pattern? (y/n): ", 'y')) {
			save_tagged_files = TRUE;
		}
	}

	if (! save_tagged_files) {
		sprintf (buf2, txt_save_pattern, default_regex_pattern);
	
		if (! parse_string (buf2, buf)) {
			return FALSE;
		}
		if (strlen (buf)) {
			my_strncpy (default_regex_pattern, buf, LEN);
		} else {
			if (default_regex_pattern[0]) {
				my_strncpy (buf, default_regex_pattern, LEN);
			} else {
				info_message (txt_no_search_string);
				return FALSE;
			}
		}
	}

	/*
	 *  ask user for filename
	 */
	if (! parse_string (txt_base_file_name, file)) {
		return FALSE;
	}

	if (file[0] == '\0') {
		info_message (txt_no_filename);
		return FALSE;
	}

	if (is_mailbox = create_path (file)) {
		if ((int) strlen (file) > 1) {
			my_strncpy (mailbox, file, LEN);
		} else {
			my_strncpy (mailbox, group, LEN);
		}
		my_strncpy (file, mailbox, LEN);
	}

	if (save_tagged_files) {
		for (i=1 ; i <= num_of_tagged_files ; i++) {
			for (j=0 ; j < top ; j++) {
				if (arts[j].tagged && arts[j].tagged == i) { 
					add_to_save_list (j, &arts[j], is_mailbox, file);
				}
			}
		}
	} else {
		for (i=0 ; i < top ; i++) {
#ifdef DONT_USE_REGEX 
			if (str_str (arts[i].subject, buf) != 0) {
#else		
			if (wildmat (arts[i].subject, buf)) {
#endif		
				add_to_save_list (i, &arts[i], is_mailbox, file);
			}
		}
		sort_save_list ();
	}

	for (i=0 ; i < save_num ; i++) {
		MoveCursor(LINES, 0);
		fputs(txt_saving, stdout);
		printf("\b\b\b\b%4d", i+1);
		fflush(stdout);

		if (is_mailbox) {
			strcpy (file, mailbox);
		}else {
			sprintf (file, "%s.%02d", save[i].file, i+1);
		}

		open_note (arts[save[i].index].artnum, group_path);
		save_art_to_file (save[i].index, i, is_mailbox, file);
		note_cleanup ();			
		save[i].saved = TRUE;
	}

	if (! save_num) {	
		info_message(txt_no_match);
	} else {
		if (is_mailbox) {
			sprintf (buf, txt_saved_to_mailbox, get_first_savefile ());
		} else {
			sprintf (buf,txt_saved_pattern_to,
				get_first_savefile (), get_last_savefile ());
		}
		info_message(buf);
	}
	
	return (mark_saved_read ? TRUE : FALSE);
}


int append_to_existing_file (i)
	int i;
{
	char buf[LEN+1];
	char *file;
	struct stat st;

	if (! save[i].is_mailbox && save_separate) {
		file = save_filename (i);
		if (stat(file, &st) != -1) {	
			sprintf (buf, txt_append_to_file, file); 
			if (! prompt_yn (LINES, buf, 'n')) {
				free (file);
				file = NULL;
				return FALSE;
			}
		}
		free (file);
		file = NULL;
	}
	
	return TRUE;
}


int create_path (path)
	char *path;
{
	char buf [LEN];
	int i, j, len;
	struct stat st;
	
	/*
	 * save in mailbox format to ~/Mail/<group.name>
	 */
	if (path[0] == '=') {
		return TRUE;
	}

	/*
	 * if ~/file expand (ie. /usr/homedir/file)
	 */
	if (path[0] == '~') {
		strcpy (buf, path+1);
		sprintf (path, "%s%s", homedir, buf);
	}

	/*
	 *  create any directories, otherwise check
	 *  errno and give appropiate error message
	 */
	len = (int) strlen (path);
	
	for (i=0, j=0 ; i < len ; i++, j++) {
		buf[j] = path[i];
		if (i+1 < len && path[i+1] == '/') {
			buf[j+1] = '\0';
			if (stat (buf, &st) == -1) {
				if (mkdir (buf, 0755) == -1) {
					error_message ("Cannot create ", buf);
					return FALSE;
				}
			}
		}
	}
	return FALSE;
}


int create_sub_dir (i)
	int i;
{
	char dir[LEN+1];
	struct stat st;

	if (! save[i].is_mailbox && save[i].archive) {
		sprintf (dir, "%s/%s", save[i].dir, save[i].archive);
		if (stat (dir, &st) == -1) {
			mkdir (dir, 0755);
			return TRUE;
		}
		if ((st.st_mode & S_IFMT) == S_IFDIR) {
			return TRUE;
		} else {
			return FALSE;
		}
	}
	return FALSE;
}

/*
 *  add files to be saved to save array
 */

void add_to_save_list (index, article, is_mailbox, path)
	int index;
	struct header *article;
	int is_mailbox;
	char *path;
{
	char dir[LEN+1];
	char file[LEN+1];
	int i;
	
	dir[0] = '\0';
	file[0] = '\0';

	if (save_num == max_save-1) {
		expand_save ();
	}

	save[save_num].index   = index;
	save[save_num].saved   = FALSE;
	save[save_num].is_mailbox = is_mailbox;
	save[save_num].dir     = (char *) 0;
	save[save_num].file    = (char *) 0;
	save[save_num].archive = (char *) 0;
	save[save_num].part    = (char *) 0;
	save[save_num].patch   = (char *) 0;

	save[save_num].subject = str_dup (article->subject);
	if (article->archive) {
		save[save_num].archive = str_dup (article->archive);
	}
	if (article->part) {
		save[save_num].part = str_dup (article->part);
	}
	if (article->patch) {
		save[save_num].patch = str_dup (article->patch);
	}

	if (is_mailbox) {
		if ((int) strlen (path) > 1) {
			if (path[0] == '=') {
				strcpy (file, path+1);
			} else {
				strcpy (file, path);
			}
		} else {
			strcpy (file, glob_page_group);
		}
		/*
		 *  convert 1st letter to uppercase
		 */
		if (file[0] >= 'a' && file[0] <= 'z') {
			file[0] = file[0] - 32;
		}
		save[save_num].dir = maildir;
		save[save_num].file = str_dup (file);
	} else {
		if (path[0]) {
			for (i=strlen (path) ; i ; i--) {
				if (path[i] == '/') {
					strncpy (dir, path, i);
					dir[i] = '\0';
					strcpy (file, path+i+1);
					break;
				}
			}
		}
		
		if (dir[0]) {
			save[save_num].dir = str_dup (dir);
		} else {
			save[save_num].dir = str_dup (savedir);
		}

		if (file[0]) {
			save[save_num].file = str_dup (file);
		} else {
			if (path[0]) {
				save[save_num].file = str_dup (path);
			} else {
				save[save_num].file = save[save_num].archive;
			}
		}
	}
	save_num++;
}

/*
 *  print save array of files to be saved
 */

void sort_save_list ()
{
	qsort (save, save_num, sizeof (struct save_t), save_comp);
}

/*
 *  string comparison routine for the qsort()
 *  ie. qsort(array, 5, 32, save_comp);
 */

int save_comp (s1, s2)
	struct save_t *s1;
	struct save_t *s2;
{
	/* s1->subject less than s2->subject */
	if (strcmp (s1->subject, s2->subject) < 0) {
		return -1;
	}
	/* s1->subject greater than s2->subject */
	if (strcmp (s1->subject, s2->subject) > 0) {
		return 1;
	}
	return 0;
}


char *save_filename (i)
	int i;
{
	char *filename;

	filename = (char *) my_malloc (LEN);

	if (save[i].is_mailbox) {
		sprintf (filename, "%s/%s", save[i].dir, save[i].file);
		return (filename);
	}
	
	if (! save_separate || ! save_archive_name || (! save[i].part && ! save[i].patch)) {
		if (! save_separate || save_num == 1) {
			sprintf (filename, "%s/%s", save[i].dir, save[i].file);
		} else {
			sprintf (filename, "%s/%s.%02d", save[i].dir, save[i].file, i+1);
		}
	} else {
		if (save[i].part) {
			if (create_sub_dir (i)) {
				sprintf (filename, "%s/%s/%s.%s%s", save[i].dir, save[i].archive, save[i].archive, LONG_PATH_PART, save[i].part);
			} else {
				sprintf (filename, "%s/%s.%s%s", save[i].dir, save[i].archive, LONG_PATH_PART, save[i].part);
			}
		} else {
			if (save[i].patch) {
				if (create_sub_dir (i)) {
					sprintf (filename, "%s/%s/%s.%s%s", save[i].dir, save[i].archive, save[i].archive, LONG_PATH_PATCH, save[i].patch);
				} else {
					sprintf (filename, "%s/%s.%s%s", save[i].dir, save[i].archive, LONG_PATH_PATCH, save[i].patch);
				}
			} else {
	 	 		sprintf (filename, "%s/%s", save[i].dir, save[i].file);
			}
		}
	}

	return (filename);
}


char *get_first_savefile ()
{
	char *file;
	int i;

	for (i=0 ; i < save_num ; i++) {
		if (save[i].saved) {
			file = (char *) my_malloc (LEN);
			if (save[i].is_mailbox) {
				sprintf (file, "%s/%s", save[i].dir, save[i].file);
				return (file);
			} else {
				if (save[i].archive && save_archive_name) {
					if (save[i].part) {
						if (create_subdir) {
							sprintf (file, "%s/%s.%s%s", save[i].archive, save[i].archive, LONG_PATH_PART, save[i].part);
						} else {
							sprintf (file, "%s.%s%s", save[i].archive, LONG_PATH_PART, save[i].part);
						}
					} else {
						if (create_subdir) {
							sprintf (file, "%s/%s.%s%s", save[i].archive, save[i].archive, LONG_PATH_PATCH, save[i].patch);
						} else {
							sprintf (file, "%s.%s%s", save[i].archive, LONG_PATH_PATCH, save[i].patch);
						}
					}
				} else {
					if (! save_separate || save_num == 1) {
						sprintf (file, "%s", save[i].file);
					} else {
						sprintf (file, "%s.%02d", save[i].file, i+1);
					}
				}
				return (file);
			}
		}
	}
	return ((char *) 0);
}


char *get_last_savefile ()
{
	char *file;
	int i;
	
	for (i=save_num-1 ; i >= 0 ; i--) {
		if (save[i].saved) {
			file = (char *) my_malloc (LEN);
			if (save[i].is_mailbox) {
				sprintf (file, "%s/%s", save[i].dir, save[i].file);
				return (file);
			} else {
				if (save[i].archive && save_archive_name) {
					if (save[i].part) {
						if (create_subdir) {
							sprintf (file, "%s/%s.%s%s", save[i].archive, save[i].archive, LONG_PATH_PART, save[i].part);
						} else {
							sprintf (file, "%s.%s%s", save[i].archive, LONG_PATH_PART, save[i].part);
						}
					} else {
						if (create_subdir) {
							sprintf (file, "%s/%s.%s%s", save[i].archive, save[i].archive, LONG_PATH_PATCH, save[i].patch);
						} else {
							sprintf (file, "%s.%s%s", save[i].archive, LONG_PATH_PATCH, save[i].patch);
						}
					}
				} else {
					if (! save_separate || save_num == 1) {
						sprintf (file, "%s", save[i].file);
					} else {
						sprintf (file, "%s.%02d", save[i].file, i+1);
					}
				}
				return (file);
			}
		}
	}
	return ((char *) 0);
}


int post_process_files ()
{
	if (post_process && save_num && ! save[save_num-1].is_mailbox) {
		wait_message (txt_post_processing);

		switch (post_proc_type) {
			case POST_PROC_SH:
				post_process_sh ();
				break;
				
			case POST_PROC_UUD:
				post_process_uud (POST_PROC_UUD);
				break;

			case POST_PROC_UUD_ZOO:
				post_process_uud (POST_PROC_UUD_ZOO);
				break;

			case POST_PROC_UUD_LZH:
				post_process_uud (POST_PROC_UUD_LZH);
				break;
				
			case POST_PROC_UUD_ARC:
				post_process_uud (POST_PROC_UUD_ARC);
				break;
				
			case POST_PROC_UUD_ZIP:
				post_process_uud (POST_PROC_UUD_ZIP);
				break;

			case POST_PROC_PATCH:
				post_process_patch ();
				break;
		}
		info_message ("-- post processing completed --");
		sleep (1);
		return TRUE;
	}
	return FALSE;
}


void post_process_uud (pp)
	int pp;
{
	char s[LEN+1], t[LEN+1], u[LEN+1];
	char buf[LEN+1], *file;
	char file_out[LEN+1];
	char file_out_dir[LEN+1];
	FILE *fp_in, *fp_out;
	int i, state = INITIAL;
	int file_size = 0;
	struct stat st;
	
	t[0] = '\0';
	u[0] = '\0';

	my_strncpy (file_out_dir, save_filename (0), LEN);
	for (i=strlen(file_out_dir) ; i > 0 ; i--) {
		if (file_out_dir[i] == '/') {
			file_out_dir[i] = '\0';
			break;
		}
	}

	sprintf (file_out, "%s/tin.%05d", file_out_dir, getpid ());
	
	if ((fp_out = fopen (file_out, "a+")) == NULL) {
		error_message (txt_cannot_open, file_out);
	}


	for (i=0 ; i < save_num ; i++) {
		my_strncpy (buf, save_filename (i), LEN);
/*
		sprintf (buf, "%s%d", txt_post_processing, i);
		wait_message (buf);
*/
		if ((fp_in = fopen (buf, "r")) != NULL) {
			if (fgets (s, LEN, fp_in) == NULL) {
				fclose (fp_in);
				continue;
			}
			while (state != END) { 
				switch (state) {
					case INITIAL:
						if (! strncmp ("begin", s, 5)) {
							state = MIDDLE;
							fprintf (fp_out, "%s", s);
						}
						break;

					case MIDDLE:
						if (s[0] == 'M') {
							fprintf (fp_out, "%s", s);
						} else if (strncmp("end", s, 3)) {
							state = OFF;
						} else { /* end */
							state = END;
							if (u[0] != 'M') {
								fprintf (fp_out, "%s", u);
							}
							if (t[0] != 'M') {
							    fprintf (fp_out, "%s", t);
							}
							fprintf (fp_out, "%s\n", s);
						}
						break;

					case OFF:
						if ((s[0] == 'M') && (t[0] == 'M') && (u[0] == 'M')) {
							fprintf (fp_out, "%s", u);
							fprintf (fp_out, "%s", t);
							fprintf (fp_out, "%s", s);
							state = MIDDLE;
						} else if (! strncmp ("end", s, 3)) {
							state = END;
							if (u[0] != 'M') {
							    fprintf (fp_out, "%s", u);
							}
							if (t[0] != 'M') {
							    fprintf (fp_out, "%s", t);
							}
							fprintf (fp_out, "%s\n", s);
						}
						break;

					case END:
						break;

					default:
						fprintf (stderr, "\r\nerror: ASSERT - default state\n");
						fclose (fp_in);
						fclose (fp_out);
						unlink (file_out);
						return;
				}
				strcpy (u,t);
				strcpy (t,s);
				/*
				 *  read next line & if error goto next file in save array
				 */
				if (fgets (s, LEN, fp_in) == NULL) {
					break;
				}
			}
			fclose (fp_in);
		}
	}
	fclose (fp_out);

	/*
	 *  uudecode file
	 */
	wait_message ("Uudecoding...");
/*
	printf ("\r\nUudecoding...\r\n"); 
*/
	
	sprintf (buf, "cd %s; uudecode %s", file_out_dir, file_out); 
	if (invoke_cmd (buf)) {
		/*
		 *  sum file
		 */
		if (file = get_archive_file (file_out_dir, "*")) { 
			sprintf (buf, "%s %s", DEFAULT_SUM, file); 
			printf ("\r\n\r\nChecksum of %s...\r\n\r\n", file); 
			fflush (stdout);
			if ((fp_in = popen (buf, "r")) == NULL) {
				printf ("Cannot execute %s\r\n", buf); 
				fflush (stdout);
			} else {
				if (stat (file, &st) != -1) {
					file_size = (int) st.st_size;
				}
				if (fgets (buf, LEN, fp_in) != NULL) {
					buf[strlen (buf)-1] = '\0';
				}
				fclose (fp_in);
				printf ("%s  %8d bytes\r\n", buf, file_size); 
				fflush (stdout);
			}
			free (file);
			file = (char *) 0;
			delete_processed_files ();
		}
	}

	if (pp > POST_PROC_UUD) {
		sprintf (buf, "*.%s", archiver[pp].ext); 
		if (file = get_archive_file (file_out_dir, buf)) {
#ifdef EXTRACT_ARCHIVED_FILES
			sprintf (buf, "cd %s; %s %s %s", file_out_dir,
				archiver[pp].name, archiver[pp].list, file);
			printf ("\r\n\r\nListing %s archive...\r\n", file); 
#else
			sprintf (buf, "cd %s; %s %s %s", file_out_dir,
				archiver[pp].name, archiver[pp].extract, file);
			printf ("\r\n\r\nExtracting %s archive...\r\n", file); 
#endif
			fflush (stdout);
			free (file);
			file = NULL;
			if (! invoke_cmd (buf)) {
				error_message ("post processing failed", NULL);
			} else {
				continue_prompt ();
			}
		}
	}

	unlink (file_out);
}

/*
 *  Unpack /bin/sh archives
 */
 
void post_process_sh ()
{
	char buf[LEN+1];
	char file_in[LEN+1];
	char file_out[LEN+1];
	char file_out_dir[LEN+1];
	char *ptr1, *ptr2;
	FILE *fp_in, *fp_out;
	int found_header;
	int i, j;
	int lineno;
	char sh_pattern_1[16];
	char sh_pattern_2[16];

	strcpy (sh_pattern_1, "#! /bin/sh");
	strcpy (sh_pattern_2, "#!/bin/sh");

	my_strncpy (file_out_dir, save_filename (0), LEN);
	for (i=strlen(file_out_dir) ; i > 0 ; i--) {
		if (file_out_dir[i] == '/') {
			file_out_dir[i] = '\0';
			break;
		}
	}

	sprintf (file_out, "%s/tin.%05d", file_out_dir, getpid ());

	for (j=0 ; j < save_num ; j++) {
		my_strncpy (file_in, save_filename (j), LEN);

		printf ("\r\nExtracting %s...\r\n", file_in);
		fflush (stdout);

		found_header = FALSE;
		lineno = 1;
		
		if ((fp_out = fopen (file_out, "w")) != NULL) {
			if ((fp_in = fopen (file_in, "r")) != NULL) {
				while (! feof (fp_in)) {
					if (fgets (buf, LEN, fp_in)) {
						/*
						 *  find #!/bin/sh or #! /bin/sh pattern
						 */
						if (!found_header) {
							ptr1 = sh_pattern_1;
							ptr2 = sh_pattern_2;
							if (str_str (buf, ptr1) != 0 ||
								str_str (buf, ptr2) != 0) {
/*
printf ("\r\n[%16s] [%04d] FOUND #/bin/sh\r\n", file_in, lineno); 
*/
								found_header = TRUE;
							}
						}
					
						/*
						 *  Write to temp file
						 */
						if (found_header) {
							fputs (buf, fp_out);
						}
						lineno++;
					}
				}
				fclose (fp_in);
			}
			fclose (fp_out);

			sprintf (buf, "cd %s; sh %s", file_out_dir, file_out); 
			printf ("\r\n");
			fflush (stdout);
			Raw (FALSE);
			invoke_cmd (buf);
			Raw (TRUE);
			unlink (file_out);
		}
	}
	delete_processed_files ();
}


char *get_archive_file (dir, ext)
	char *dir;
	char *ext;
{
	char buf[LEN+1];
	char *file = NULL;
	FILE *fp;
	
	sprintf (buf, "ls -t %s/%s", dir, ext);

	if ((fp = popen (buf, "r")) == NULL) {
		return NULL;
	}

	if (fgets (buf, LEN, fp) != NULL) {
		file = str_dup (buf);
		file[strlen (file)-1] = '\0';
	}
	
	fclose (fp);

	return (file);
}


void delete_processed_files ()
{
	int i;

	printf ("\r\n");
	fflush (stdout);
	
	if (prompt_yn (LINES, "Delete saved files that have been post processed? (y/n): ", 'y')) {
		wait_message (txt_deleting);

		for (i=0 ; i < save_num ; i++) {
			unlink (save_filename (i));
		}
	}
}


void post_process_patch ()
{
}
