/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : select.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 20-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"


int first_group_on_screen;
int last_group_on_screen;
int cur_groupnum = 0;
int reread_active_file = TRUE;
extern int index_point;
int space_mode;
extern char cvers[LEN+1];


#ifdef SIGTSTP
void select_susp (sig)
	int sig;
{

	Raw(FALSE);
	putchar('\n');
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask(sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
	kill(0, SIGTSTP);

	signal(SIGTSTP, select_susp);

	if (! update) {
		Raw(TRUE);
	}

#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	mail_setup();
	group_selection_page();
}
#endif


void selection_index()
{
	char ch;
	int i, n;
	int subscribe_num;
	char buf[LEN+1];

	group_selection_page ();		/* display group selection page */

	while (1) {
		ch = ReadCh();

		if (ch > '0' && ch <= '9') {
			prompt_group_num(ch);
		} else switch (ch) {
			case '!':
				shell_escape();
				group_selection_page();
				break;

			case '$':	/* show last page of groups */
end_of_list:			
				cur_groupnum = local_top - 1;
				group_selection_page();
			    break;

			case '/':	/* search forward */
			case '?':	/* search backward */
				i = (ch == '/');
				search_group (i);
				break;

			case '\r':	/* go into group */
			case '\n':
				space_mode = FALSE;
go_into_group:
				clear_message();
				index_point = -1;
				do {
					group_page (active[my_group[cur_groupnum]].name);
				} while (index_point == -3);
				group_selection_page();
				break;

			case '\t':
				for (i = cur_groupnum; i < local_top; i++)
					if (unread[i] != 0)
						break;
				if (i >= local_top) {
					info_message(txt_no_groups_to_read);
					break;
				}

				erase_group_arrow();
				cur_groupnum = i;
				if (cur_groupnum >= last_group_on_screen)
					group_selection_page();
				else
					draw_group_arrow();
				space_mode = TRUE;
				goto go_into_group;

			case 27:	/* (ESC) common arrow keys */
				ch = ReadCh();
				if (ch == '[' || ch == 'O')
					ch = ReadCh();
				switch (ch) {
				case 'A':
				case 'D':
				case 'i':
					goto select_up;

				case 'B':
				case 'C':
					goto select_down;

				case 'G':		/* ansi  PgDn */
				case 'U':		/* at386 PgDn */
					goto select_page_down;

				case 'I':		/* ansi  PgUp */
				case 'V':		/* at386 PgUp */
					goto select_page_up;

				case 'H':		/* at386  Home */
					cur_groupnum = 0;
					group_selection_page ();
					break;
					
				case 'F':		/* ansi  End */
				case 'Y':		/* at386  End */
					goto end_of_list;
				}
				break;

			case ctrl('D'):		/* page down */
			case ' ':
select_page_down:
				erase_group_arrow();
				cur_groupnum += NOTESLINES / 2;
				if (cur_groupnum >= local_top)
					cur_groupnum = local_top - 1;

				if (cur_groupnum <= first_group_on_screen
				||  cur_groupnum >= last_group_on_screen)
					group_selection_page();
				else
					draw_group_arrow();
				break;

			case ctrl('K'):
				if (local_top <= 0) {
					info_message(txt_no_groups_to_delete);
					break;
				}

				if (prompt_yn (LINES, txt_del_group_in_newsrc, 'y')) {
					delete_group(active[my_group[cur_groupnum]].name);
					active[my_group[cur_groupnum]].flag = NOTGOT;	

					local_top--;
					for (i = cur_groupnum; i < local_top; i++) {
						my_group[i] = my_group[i+1];
						unread[i] = unread[i+1];
					}
					if (cur_groupnum >= local_top)
						cur_groupnum = local_top - 1;	

					group_selection_page();
					info_message(txt_group_deleted);
				}
				break;

			case ctrl('L'):		/* redraw */
			case 't':
#ifndef USE_CLEARSCREEN
				ClearScreen ();
#endif
				group_selection_page();
				break;

			case ctrl('N'):		/* line down */
			case 'j':
select_down:
				if (cur_groupnum + 1 >= local_top)
					break;

				if (cur_groupnum + 1 >= last_group_on_screen) {
#ifndef USE_CLEARSCREEN
					erase_group_arrow();
#endif					
					cur_groupnum++;
					group_selection_page();
				} else {
					erase_group_arrow();
					cur_groupnum++;
					draw_group_arrow();
				}
				break;

			case ctrl('P'):		/* line up */
			case 'k':
select_up:
				if (!cur_groupnum)
					break;

				if (cur_groupnum <= first_group_on_screen) {
					cur_groupnum--;
					group_selection_page();
				} else {
					erase_group_arrow();
					cur_groupnum--;
					draw_group_arrow();
				}
				break;

			case ctrl('R'):	/* reset .newsrc */
			    if (prompt_yn (LINES, txt_reset_newsrc, 'n')) {
					reset_newsrc();
					cur_groupnum = 0;
					group_selection_page();
			    }
			    break;

			case ctrl('U'):		/* page up */
			case 'b':
select_page_up:
				erase_group_arrow();
				cur_groupnum -= NOTESLINES / 2;
				if (cur_groupnum < 0)
					cur_groupnum = 0;
				if (cur_groupnum < first_group_on_screen
				||  cur_groupnum >= last_group_on_screen)
					group_selection_page();
				else
					draw_group_arrow();
				break;

			case 'B':	/* bug/gripe/comment mailed to author */
				mail_bug_report ();
#ifndef USE_CLEARSCREEN
				ClearScreen ();
#endif
				group_selection_page();
				break;
				
			case 'c':	/* catchup--mark all articles as read */
				if (prompt_yn (LINES, txt_mark_group_read, 'y')) {
					unread[cur_groupnum] = 0;
					mark_group_read(
						active[my_group[cur_groupnum]].name,
						my_group[cur_groupnum]);
					if (draw_arrow_mark) {	
						MoveCursor (INDEX_TOP +
						    (cur_groupnum - first_group_on_screen), 47);
						printf ("     ");
						MoveCursor (LINES, 0);
						fflush (stdout);
					} else {
						i = cur_groupnum - first_group_on_screen;
						screen[i].col[47] = ' ';
						screen[i].col[48] = ' ';
						screen[i].col[49] = ' ';
						screen[i].col[50] = ' ';
					}
					erase_group_arrow ();
					if (cur_groupnum+1 < last_group_on_screen) {
						cur_groupnum++;
					}
					draw_group_arrow ();
			    }
			    break;

			case 'g':	/* prompt for a new group name */
				if ((n = choose_new_group ()) >= 0) {
					if (active[my_group[n]].flag != SUBS) {
						subscribe (active[my_group[n]].name, ':',
							my_group[n], FALSE);
					}
					erase_group_arrow();
					cur_groupnum = reposition_group (active[my_group[n]].name,
													(n ? n : cur_groupnum));
					if (cur_groupnum < first_group_on_screen ||
						cur_groupnum >= last_group_on_screen ||
						cur_groupnum != n) {
						group_selection_page();
					} else {
						clear_message ();
						draw_group_arrow();
					}
				}
				break;

			case 'h':
				help_select_commands ();
				group_selection_page ();
				break;

			case 'H':
				help_select_info ();
				group_selection_page ();
				break;

			case 'I':		/* toggle inverse video */
				inverse_okay = !inverse_okay;
				if (inverse_okay)
					info_message(txt_inverse_on);
				else
					info_message(txt_inverse_off);
				group_selection_page();
				break;

			case 'm':	/* reposition group within group list */
				if (active[my_group[cur_groupnum]].flag == SUBS) {
					n = cur_groupnum;
					erase_group_arrow ();
					cur_groupnum = reposition_group (active[my_group[n]].name, n);
					if (cur_groupnum < first_group_on_screen ||
						cur_groupnum >= last_group_on_screen ||
						cur_groupnum != n) {
						group_selection_page();
					} else {
						clear_message ();
						draw_group_arrow();
					}
				}
			    break;

			case 'M':	/* options menu */
				change_rcfile ("", TRUE);
				group_selection_page ();
			    break;

			case 'q':	/* quit */
				tin_done (0);

			case 's':	/* subscribe to current group */
			    if (active[my_group[cur_groupnum]].flag != SUBS) {
				    MoveCursor (INDEX_TOP + (cur_groupnum-first_group_on_screen), 3);
					if (draw_arrow_mark) {
						putchar (' ');
					} else {
						screen[cur_groupnum-first_group_on_screen].col[3] = ' ';
						draw_group_arrow ();
					}
					fflush (stdout);
					MoveCursor (LINES, 0);	

					subscribe (active[my_group[cur_groupnum]].name,
						':', my_group[cur_groupnum], FALSE);
					sprintf (buf, txt_subscribed_to, active[my_group[cur_groupnum]].name);
					info_message (buf);
				}
			    break;

			case 'S':	/* subscribe to groups matching pattern */
				if (parse_string (txt_subscribe_pattern, buf) && buf[0]) {
					wait_message (txt_subscribing);
					for (subscribe_num=0, i=0 ; i < local_top ; i++) {
#ifdef DONT_USE_REGEX 
						if (str_str (active[my_group[i]].name, buf)) {
#else		
						if (wildmat (active[my_group[i]].name, buf)) {
#endif		
			   		 		if (active[my_group[i]].flag != SUBS) {
								sprintf (msg, txt_subscribing_to, active[my_group[i]].name);
								wait_message (msg);
								subscribe (active[my_group[i]].name,
									':', my_group[i], FALSE);
							}
							subscribe_num++;
						}
					}
					if (subscribe_num) {
						group_selection_page ();	
						sprintf (buf, txt_subscribed_num_groups, subscribe_num);
						info_message (buf);
					} else {
						info_message (txt_no_match);
					}
				} else {
					clear_message ();
				}
			    break;

			case 'u':	/* unsubscribe to current group */
			    if (active[my_group[cur_groupnum]].flag == SUBS) {
				    MoveCursor(INDEX_TOP + (cur_groupnum-first_group_on_screen), 3);
					if (draw_arrow_mark) {
						putchar('u');
					} else {
						screen[cur_groupnum-first_group_on_screen].col[3] = 'u';
						draw_group_arrow ();
					}
					fflush(stdout);
					MoveCursor(LINES, 0);

					subscribe(active[my_group[cur_groupnum]].name,
						'!', my_group[cur_groupnum], FALSE);
				    sprintf(buf, txt_unsubscribed_to,active[my_group[cur_groupnum]].name);
				    info_message(buf);
				}
				break;

			case 'U':	/* unsubscribe to groups matching pattern */
				if (parse_string (txt_unsubscribe_pattern, buf) && buf[0]) {	
					wait_message (txt_unsubscribing);
					for (subscribe_num=0, i=0 ; i < local_top ; i++) {		
#ifdef DONT_USE_REGEX 
						if (str_str (active[my_group[i]].name, buf)) {
#else		
						if (wildmat (active[my_group[i]].name, buf)) {
#endif		
			   		 		if (active[my_group[i]].flag == SUBS) {
								sprintf (msg, txt_unsubscribing_from, active[my_group[i]].name);
								wait_message (msg);
								subscribe (active[my_group[i]].name,
									'!', my_group[i], FALSE);
							}
							subscribe_num++;
						}
					}
					if (subscribe_num) {
						group_selection_page ();	
						sprintf (buf, txt_unsubscribed_num_groups, subscribe_num);
						info_message (buf);
					} else {
						info_message (txt_no_match);
					}
				} else {
					clear_message ();
				}
			    break;

			case 'v':	/* show tin version */
				info_message (cvers);
				break;

			case 'W':	/* display messages posted by user */
				if (user_posted_messages ()) {
					group_selection_page ();
				}
				break;

			case 'y':	/* pull in rest of groups from active */
				if (reread_active_file) {
					n = local_top;
					for (i = 0; i < num_active; i++) {
						active[i].flag = NOTGOT;
					}
					read_newsrc (FALSE);
					for (i = 0; i < num_active; i++) {
						if (active[i].flag & NOTGOT) {
							active[i].flag &= ~NOTGOT;
							my_group[local_top] = i;
							unread[local_top] = -1;
							local_top++;
						}
					}
					if (n < local_top) {
						sprintf(buf, txt_added_groups, local_top - n,
							local_top - n == 1 ? "" : txt_plural);
						group_selection_page();
						info_message(buf);
					} else {
					    info_message(txt_no_groups_to_yank_in);
					}
					reread_active_file = FALSE;
				} else {
					read_newsrc (TRUE);
					cur_groupnum = local_top - 1;
					group_selection_page();
					reread_active_file = TRUE;
				}
				break;

			case 'Y':	/* reread .newsrc, no unsub groups */
				cur_groupnum = 0;
				local_top = 0;
				for (i = 0; i < num_active; i++)
					active[i].flag = NOTGOT;
				read_newsrc(TRUE);
				group_selection_page();
				break;

			case 'z':
				undel_group();
				group_selection_page();
				break;

			default:
			    info_message(txt_bad_command);
		}
	}
}


void group_selection_page ()
{
	int i,j;
	int n;
	char new[10];
	char subs;

#ifdef SIGTSTP
	signal(SIGTSTP, select_susp);
#endif

#ifdef USE_CLEARSCREEN
	ClearScreen ();
#else
	MoveCursor (0, 0);		/* top left corner */
	CleartoEOLN ();
#endif

	printf("%s\r\n", nice_time());		/* print time in upper left */

	MoveCursor (0, (COLS - (int) strlen (txt_type_h_for_help))+1);	/* in upper middle */
	if (mail_check ()) {			/* you have mail message */
		printf(txt_you_have_mail);
	} else {
		printf(txt_type_h_for_help);
	}

#ifndef USE_CLEARSCREEN
	MoveCursor (1, 0);
	CleartoEOLN ();
	MoveCursor (2, 0);
	CleartoEOLN ();
#endif

	StartInverse();
	center_line(1, txt_group_selection);
	EndInverse();
	
	MoveCursor(1, (COLS - (int) strlen (txt_type_h_for_help))+1);	/* in upper middle */
	if (kill_articles) {		/* display KILL on screen */
		printf("KILL ON   ");
	} else {
		printf("          ");
	}
	if (post_process) {		/* display POST on screen */
		printf("POST ON");
	} else {
		printf("       ");
	}
		
	MoveCursor(INDEX_TOP, 0);

	first_group_on_screen = (cur_groupnum / NOTESLINES) * NOTESLINES;

	last_group_on_screen = first_group_on_screen + NOTESLINES;
	if (last_group_on_screen >= local_top)
		last_group_on_screen = local_top;

	for (j=0, i = first_group_on_screen; i < last_group_on_screen; i++,j++) {
		switch (unread[i]) {
			case -2:
				sprintf(new, "?   ");
				break;

			case -1:
				sprintf (new, "-   ");
				break;

			case 0:
				sprintf (new, "    ");
				break;

			default:
				sprintf (new, "%-4d", unread[i]);
		}
		
		n = my_group[i];
		if (active[n].flag & SUBS)	/* subscribed? */
			subs = ' ';
		else
			subs = 'u';	/* u next to unsubscribed groups */

		if (draw_arrow_mark) {
			printf ("   %c %4d  %-35s %s\r\n",
			       subs, i+1, active[n].name, new);
		} else {
			sprintf (screen[j].col, "   %c %4d  %-35s %s% *s\r\n",
				subs, i+1, active[n].name, new, COLS-52, " ");
			printf ("%s", screen[j].col);
		}
	}
#ifndef USE_CLEARSCREEN
	CleartoEOS ();
#endif

	draw_group_arrow();
}


int prompt_group_num(ch)
	char ch;
{
	int num;

	clear_message();

	if ((num = parse_num(ch, txt_select_group)) == -1) {
		clear_message();
		return FALSE;
	}
	num--;		/* index from 0 (internal) vs. 1 (user) */

	if (num >= local_top)
		num = local_top - 1;

	if (num >= first_group_on_screen
	&&  num < last_group_on_screen) {
		erase_group_arrow();
		cur_groupnum = num;
		draw_group_arrow();
	} else {
#ifndef USE_CLEARSCREEN
		erase_group_arrow();
#endif		
		cur_groupnum = num;
		group_selection_page();
	}

	return TRUE;
}


void erase_group_arrow()
{
	erase_arrow(INDEX_TOP + (cur_groupnum-first_group_on_screen) );
}


void draw_group_arrow()
{
	draw_arrow(INDEX_TOP + (cur_groupnum-first_group_on_screen) );
}


int choose_new_group ()
{
	char buf[LEN+1];
	char *p;
	int ret;

	if (! parse_string (txt_newsgroup, buf))
		return -1;

	for (p = buf; *p && (*p == ' ' || *p == '\t'); p++) ;
	if (*p == '\0')
		return -1;

	clear_message ();

	if ((ret = add_group (p, TRUE)) < 0) {
		sprintf (msg, txt_not_in_active_file, p);
		info_message (msg);
	}

	return ret;
}


/*
 *  Add a group to the selection list (my_group[])
 *  Return the index of my_group[] if group is added or was already
 *  there.  Return -1 if named group is not in active[].
 */

int add_group (s, get_unread)
	char *s;
	int get_unread;			/* look in .newsrc for sequencer unread info? */
{
	long h;
	int i, j;

	h = hash_groupname (s);

	for (i = group_hash[h]; i >= 0; i = active[i].next) {
		if (strcmp (s, active[i].name) == 0) {
			for (j = 0; j < local_top; j++) {
				if (my_group[j] == i) {
					return j;
				}
			}

			active[i].flag &= ~NOTGOT;   /* mark that we got it */
			my_group[local_top] = i;

			if (get_unread)
				unread[local_top] = get_line_unread (s, i);
			else
				unread[local_top] = -2;

			local_top++;
			return local_top - 1;
		}
	}

	return -1;
}

/*
 *  Find the next unread response in this group 
 */

int next_unread(n)
	int n;
{
	while (n >= 0) {
		if (arts[n].unread == ART_UNREAD)
			return n;
		n = next_response (n);
	}

	return -1;
}


/*
 *  Find the previous unread response in this thread
 */

int prev_unread(n)
	int n;
{
	while (n >= 0) {
		if (arts[n].unread == ART_UNREAD)
			return n;
		n = prev_response(n);
	}

	return -1;
}


int reposition_group (group, default_num)
	char *group;
	int default_num;
{
	char buf[LEN+1];
	char pos[LEN+1];
	int pos_num = 0;

	sprintf (buf, txt_newsgroup_position, group);
	
	if (! parse_string (buf, pos)) 
		return default_num;

	if (pos[0] == '\0')
		return default_num;
		
	if (pos[0] == '$') {
		pos_num = local_top;
	} else {
		pos_num = atoi (pos);
		if (pos_num > local_top) {
			pos_num = local_top;
		} else if (pos_num <= 0) {
			pos_num = 1;
		}
	}

	wait_message (txt_moving);
	
	if (pos_group_in_newsrc (group, pos_num)) {
		read_newsrc (TRUE);
		return (pos_num-1);
	} else {
		return (default_num);
	}
}
