/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : tin.h
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 20-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	<stdio.h>
#include	<signal.h>
#include	<pwd.h>
#include	<sys/types.h>
#include	<memory.h>
#include	<ctype.h>
#include	<time.h>
#include	<sys/stat.h>
#ifdef BSD
#include	<strings.h>
#else
#include	<string.h>
#include	<malloc.h>
#endif /* BSD */
#ifdef SCO_UNIX
#include	<sys/streams.h>
#include	<sys/ptem.h>
#endif /* SCO_UNIX */

#define BUG_REPORT_ADDRESS	"iain@estevax.uucp"

#ifndef LIBDIR
#define		LIBDIR		"/usr/lib/news"
#endif
#ifndef SPOOLDIR
#define		SPOOLDIR	"/usr/spool/news"
#endif

#define		RCDIR		".tin"
#define		RCFILE		"tinrc"
#define		INDEXDIR	"index"
#define		KILLFILE	"kill"
#define		POSTFILE	"posted"

#define		DEFAULT_SHELL	"/bin/sh"
#define		DEFAULT_MAILBOX	"Mail"

#ifdef BSD
#define     DEFAULT_EDITOR	"/usr/ucb/vi"
#define		DEFAULT_MAILER	"/bin/rmail"
#define		DEFAULT_PRINTER	"/usr/ucb/lpr"
#define		DEFAULT_SUM		"sum"
#ifndef USE_LONG_FILENAMES
#define USE_LONG_FILENAMES
#endif
#else
#define     DEFAULT_EDITOR	"/usr/bin/vi"
#define		DEFAULT_MAILER	"/usr/bin/rmail"
#define		DEFAULT_PRINTER	"/usr/bin/lp"
#define		DEFAULT_SUM		"sum -r"
#endif

#define		DEFAULT_ACTIVE_NUM	100		/* initial size of active array */
#define		DEFAULT_ARTICLE_NUM	400		/* initial size of art array */ 
#define		DEFAULT_KILL_NUM	10		/* initial size of kill array */ 
#define		DEFAULT_SAVE_NUM	10		/* initial size of save array */ 

#define		TRUE		1
#define		FALSE		0

#define		LEN		200

#ifdef USE_LONG_FILENAMES
#define		LONG_PATH_PART	"part"
#define		LONG_PATH_PATCH	"patch"
#else
#define		LONG_PATH_PART	""
#define		LONG_PATH_PATCH	"p"
#endif

#define		INDEX_TOP		3
#define		NOTESLINES		(LINES - INDEX_TOP - 1)
#define		RIGHT_POS		(COLS - 16)
#define		MORE_POS		(COLS - 15)

#define		SORT_BY_NONE	0		/* sort types on arts[] array */
#define		SORT_BY_SUBJ	1
#define		SORT_BY_FROM	2
#define		SORT_BY_DATE	3

extern int sort_art_type;

extern int max_subj;
extern int max_from;

#define		MAX_ARCH		38
#define		TABLE_SIZE		1409

#define		MAX_PAGES		1000

#ifdef BSD
#define CR	'\r'
#else
#define CR	10
#endif

/*
 * return codes for change_rcfile ()
 */

#define		NO_KILLING		0
#define		KILLING			1

/*
 *  art.thread
 */

#define		ART_NORMAL		-1
#define		ART_EXPIRED		-2

/*
 *  art.unread
 */

#define		ART_READ		0
#define		ART_UNREAD		1
#define		ART_WILL_RETURN	2

struct header {
	long artnum;
	char *subject;	/* Subject: line from mail header */
	char *from;		/* From: line from mail header */
	int thread;
	int inthread;
	int unread;		/* 0 = read, 1 = unread, 2 = will return */
	char date[16];	/* Date: line from mail header */
	char *archive;	/* Archive-name: line from mail header */
	char *part;		/* part no. of archive */
	char *patch;	/* patch no. of archive */
	int tagged;		/* 0 = not killed/tagged, 1 = killed/tagged */
};

/*
 *  header.artnum:
 *	article number in spool directory for group
 *
 *  header.thread:
 *	initially -1
 *	points to another arts[] (struct header): zero and up
 *	-2 means article has expired (wasn't found in file search
 *	of spool directory for the group)
 *
 *  header.inthread:
 *	FALSE for the first article in a thread, TRUE for all
 *	following articles in thread
 *
 *  header.read:
 *	boolean, has this article been read or not
 *
 *  header.date
 *  date: line used for sorting articles by date order
 *
 *  header.archive:
 *	archive name used in *source* groups
 *
 *  header.part:
 *	part no. of archive
 *
 *  header.patch:
 *	patch no. of archive
 *
 *  header.tagged:
 *	boolean, has this article been killed/tagged for saving or not
 */

struct group_ent {
	char *name;
	long max;
	long min;
	int next;		/* next active entry in hash chain */
	int flag;
};

#define		NOTGOT			0x01	/* haven't put in my_group yet */
#define		SUBS			0x02	/* subscribed to */

#define KILL_SUBJ	1
#define KILL_FROM	2
#define KILL_BOTH	3

struct kill_t {
	int kill_type;
	long kill_group;
	char *kill_subj;
	char *kill_from;
};

extern struct kill_t *killf;
extern int kill_num;
extern int max_kill;

#define POST_PROC_SH		1
#define POST_PROC_UUD		2
#define POST_PROC_UUD_ZOO	3
#define POST_PROC_UUD_LZH	4
#define POST_PROC_UUD_ARC	5
#define POST_PROC_UUD_ZIP	6
#define POST_PROC_PATCH		7

struct save_t {
	char *subject;
	char *dir;
	char *file;
	char *archive;
	char *part;
	char *patch;
	int index;	
	int saved;	
	int is_mailbox;	
};

extern struct save_t *save;
extern int save_num;
extern int max_save;

struct screen_t {
	char *col;
};

extern struct screen_t *screen;

extern int debug;
extern int active_num;
extern int article_num;
extern int start_line_offset;

extern int read_news_via_nntp;
extern char nntp_server[LEN+1];
extern int top;
extern struct header *arts;
extern long *base;
extern int max_art;

extern int index_file_killed;
extern int save_separate;
extern int save_archive_name;
extern int show_author;
extern int draw_arrow_mark;
extern int kill_articles;
extern int post_process;
extern int mark_saved_read;
extern int post_proc_type;
extern int default_printer;
extern int print_header;
extern int num_of_tagged_files;

extern char rcdir[LEN+1];
extern char rcfile[LEN+1];
extern char indexdir[LEN+1];
extern char killfile[LEN+1];
extern char postfile[LEN+1];

extern char msg[LEN+1];
extern char userid[LEN+1];
extern char homedir[LEN+1];
extern char add_addr[LEN+1];
extern char reply_to[LEN+1];
extern char my_org[LEN+1];
extern char active_file[LEN+1];
extern char newsrc[LEN+1];
extern char newnewsrc[LEN+1];
extern char delgroups[LEN+1];
extern int top_base;
extern int LINES, COLS;
extern int group_hash[TABLE_SIZE];

extern char *version;
extern char cvers[LEN+1];
extern char progname[LEN+1];
extern char mailer[LEN+1];
extern char printer[LEN+1];
extern char spooldir[LEN+1];
extern char savedir[LEN+1];
extern char maildir[LEN+1];
extern char page_header[LEN+1];
extern char signature[LEN+1];
extern char sig[LEN+1];
extern char killsubj[LEN+1];
extern char killfrom[LEN+1];
extern char author_search_string[LEN+1];
extern char group_search_string[LEN+1];
extern char subject_search_string[LEN+1];
extern char art_search_string[LEN+1];
extern char default_regex_pattern[LEN+1];

extern int num_active;
extern struct group_ent *active;
extern int *my_group;
extern int *unread;
extern int max_active;

extern int local_top;
extern int catchup;
extern int update;
extern int verbose;
extern int inverse_okay;

extern int tin_uid;
extern int tin_gid;
extern int real_uid;
extern int real_gid;
extern int local_index;

extern char *strcpy();
extern char *strncat();
extern char *strncpy();
extern long atol();


#define		ctrl(c)			((c) & 0x1F)

/*
 *  Assertion verifier
 */

#ifdef __STDC__
#define	assert(p)	if(! (p)) asfail(__FILE__, __LINE__, #p); else
#else
#define	assert(p)	if(! (p)) asfail(__FILE__, __LINE__, "p"); else
#endif

/*
 *  function prototypes	& language definitions	
 */ 

#include	"proto.h"
#include	"lang.h"
